@@ -1,5 +1,4 @@
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -11,7 +10,6 @@
 
 #include <mtx/responses/common.hpp>
 
-#include "Cache.h"
 #include "Cache_p.h"
 #include "ChatPage.h"
 #include "Logging.h"
@@ -43,7 +41,7 @@ AliasEditingModel::AliasEditingModel(const std::string &rid, QObject *parent)
         }
     }
 
-    for (const auto &alias : qAsConst(aliases)) {
+    for (const auto &alias : std::as_const(aliases)) {
         fetchAliasesStatus(alias.alias);
     }
     fetchPublishedAliases();
@@ -149,7 +147,7 @@ void
 AliasEditingModel::addAlias(QString newAlias)
 {
     const auto aliasStr = newAlias.toStdString();
-    for (const auto &e : qAsConst(aliases)) {
+    for (const auto &e : std::as_const(aliases)) {
         if (e.alias == aliasStr) {
             return;
         }
@@ -335,3 +333,5 @@ AliasEditingModel::commit()
           }
       });
 }
+
+#include "moc_AliasEditModel.cpp"
