@@ -1,17 +1,15 @@
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
 #include <QAbstractListModel>
+#include <QQmlEngine>
 #include <QVector>
 
 #include <mtx/events/canonical_alias.hpp>
 
-#include "CacheStructs.h"
-
 class FetchPublishedAliasesJob final : public QObject
 {
     Q_OBJECT
@@ -30,6 +28,9 @@ signals:
 class AliasEditingModel final : public QAbstractListModel
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_UNCREATABLE("Please use editAliases to create the models")
+
     Q_PROPERTY(bool canAdvertize READ canAdvertize CONSTANT)
 
 public:
