@@ -1,13 +1,9 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "BlurhashProvider.h"
 
-#include <algorithm>
-
 #include <QUrl>
 
 #include "blurhash.hpp"
@@ -26,9 +22,14 @@ BlurhashRunnable::run()
         return;
     }
 
+    auto blurhashDecodeSize = m_requestedSize;
+    if (blurhashDecodeSize.height() > 100 && blurhashDecodeSize.width() > 100) {
+        blurhashDecodeSize.scale(100, 100, Qt::AspectRatioMode::KeepAspectRatio);
+    }
+
     auto decoded = blurhash::decode(QUrl::fromPercentEncoding(m_id.toUtf8()).toStdString(),
-                                    m_requestedSize.width(),
-                                    m_requestedSize.height());
+                                    blurhashDecodeSize.width(),
+                                    blurhashDecodeSize.height());
     if (decoded.image.empty()) {
         emit error(QStringLiteral("Failed decode!"));
         return;
@@ -40,5 +41,9 @@ BlurhashRunnable::run()
                  (int)decoded.width * 3,
                  QImage::Format_RGB888);
 
+    image = image.scaled(m_requestedSize);
+
     emit done(image.convertToFormat(QImage::Format_RGB32));
 }
+
+#include "moc_BlurhashProvider.cpp"
