@@ -1,7 +1,4 @@
-// SPDX-FileCopyrightText: 2017 Konstantinos Sideris <siderisk@auth.gr>
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -29,6 +26,9 @@ struct Notifications;
 }
 
 namespace cache {
+void
+setNeedsCompactFlag();
+
 void
 init(const QString &user_id);
 
@@ -253,7 +253,10 @@ std::string
 restoreOlmAccount();
 
 void
-storeSecret(const std::string &name, const std::string &secret);
+storeSecret(std::string_view name, const std::string &secret);
 std::optional<std::string>
-secret(const std::string &name);
+secret(std::string_view name);
+
+std::vector<ImagePackInfo>
+getImagePacks(const std::string &room_id, std::optional<bool> stickers);
 }
