@@ -1,14 +1,9 @@
-// SPDX-FileCopyrightText: 2017 Konstantinos Sideris <siderisk@auth.gr>
-// SPDX-FileCopyrightText: 2019 The nheko authors
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
-#include <limits>
 #include <optional>
 
 #include <QDateTime>
@@ -19,19 +14,19 @@
 #else
 #include <lmdb++.h>
 #endif
-#include <nlohmann/json.hpp>
 
+#include <mtx/events/collections.hpp>
 #include <mtx/responses/notifications.hpp>
 #include <mtx/responses/sync.hpp>
 #include <mtxclient/crypto/types.hpp>
-#include <mtxclient/http/client.hpp>
+#include <mtxclient/http/errors.hpp>
 
 #include "CacheCryptoStructs.h"
 #include "CacheStructs.h"
-#include "Logging.h"
 
 namespace mtx::responses {
 struct Messages;
+struct StateEvents;
 }
 
 class Cache final : public QObject
@@ -57,8 +52,9 @@ public:
                                lmdb::dbi &db,
                                const std::vector<std::string> &user_ids,
                                const std::string &sync_token);
-    void query_keys(const std::string &user_id,
-                    std::function<void(const UserKeyCache &, mtx::http::RequestErr)> cb);
+    void query_keys(
+      const std::string &user_id,
+      std::function<void(const UserKeyCache &, const std::optional<mtx::http::ClientError> &)> cb);
 
     // device & user verification cache
     std::optional<UserKeyCache> userKeys(const std::string &user_id);
@@ -68,6 +64,7 @@ public:
     crypto::Trust roomVerificationStatus(const std::string &room_id);
 
     std::vector<std::string> joinedRooms();
+    std::map<std::string, RoomInfo> getCommonRooms(const std::string &user_id);
 
     QMap<QString, RoomInfo> roomInfo(bool withInvites = true);
     QHash<QString, RoomInfo> invites();
@@ -87,6 +84,15 @@ public:
     QString getRoomVersion(lmdb::txn &txn, lmdb::dbi &statesdb);
     //! Retrieve if the room is a space
     bool getRoomIsSpace(lmdb::txn &txn, lmdb::dbi &statesdb);
+    //! Retrieve if the room is tombstoned (closed or replaced by a different room)
+    bool getRoomIsTombstoned(lmdb::txn &txn, lmdb::dbi &statesdb);
+
+    // for the event expiry background job
+    void storeEventExpirationProgress(const std::string &room,
+                                      const std::string &expirationSettings,
+                                      const std::string &stopMarker);
+    std::string
+    loadEventExpirationProgress(const std::string &room, const std::string &expirationSettings);
 
     //! Get a specific state event
     template<typename T>
@@ -184,26 +190,14 @@ public:
     //! Check if we have sent a desktop notification for the given event id.
     bool isNotificationSent(const std::string &event_id);
 
-    //! retrieve events in timeline and related functions
-    struct Messages
-    {
-        mtx::responses::Timeline timeline;
-        uint64_t next_index;
-        bool end_of_cache = false;
-    };
-    Messages getTimelineMessages(lmdb::txn &txn,
-                                 const std::string &room_id,
-                                 uint64_t index = std::numeric_limits<uint64_t>::max(),
-                                 bool forward   = false);
-
-    std::optional<mtx::events::collections::TimelineEvent>
+    std::optional<mtx::events::collections::TimelineEvents>
     getEvent(const std::string &room_id, std::string_view event_id);
     void storeEvent(const std::string &room_id,
                     const std::string &event_id,
-                    const mtx::events::collections::TimelineEvent &event);
+                    const mtx::events::collections::TimelineEvents &event);
     void replaceEvent(const std::string &room_id,
                       const std::string &event_id,
-                      const mtx::events::collections::TimelineEvent &event);
+                      const mtx::events::collections::TimelineEvents &event);
     std::vector<std::string> relatedEvents(const std::string &room_id, const std::string &event_id);
 
     struct TimelineRange
@@ -222,9 +216,9 @@ public:
     std::string previousBatchToken(const std::string &room_id);
     uint64_t saveOldMessages(const std::string &room_id, const mtx::responses::Messages &res);
     void savePendingMessage(const std::string &room_id,
-                            const mtx::events::collections::TimelineEvent &message);
+                            const mtx::events::collections::TimelineEvents &message);
     std::vector<std::string> pendingEvents(const std::string &room_id);
-    std::optional<mtx::events::collections::TimelineEvent>
+    std::optional<mtx::events::collections::TimelineEvents>
     firstPendingMessage(const std::string &room_id);
     void removePendingStatus(const std::string &room_id, const std::string &txn_id);
 
@@ -299,9 +293,9 @@ public:
     void deleteBackupVersion();
     std::optional<OnlineBackupVersion> backupVersion();
 
-    void storeSecret(const std::string &name, const std::string &secret, bool internal = false);
-    void deleteSecret(const std::string &name, bool internal = false);
-    std::optional<std::string> secret(const std::string &name, bool internal = false);
+    void storeSecret(std::string_view name, const std::string &secret, bool internal = false);
+    void deleteSecret(std::string_view name, bool internal = false);
+    std::optional<std::string> secret(std::string_view name, bool internal = false);
 
     std::string pickleSecret();
 
@@ -313,9 +307,12 @@ public:
     static int compare_state_key(const MDB_val *a, const MDB_val *b)
     {
         auto get_skey = [](const MDB_val *v) {
-            return nlohmann::json::parse(
-                     std::string_view(static_cast<const char *>(v->mv_data), v->mv_size))
-              .value("key", "");
+            auto temp = std::string_view(static_cast<const char *>(v->mv_data), v->mv_size);
+            // allow only passing the state key, in which case no null char will be in it and we
+            // return the whole string because rfind returns npos.
+            // We search from the back, because state keys could include nullbytes, event ids can
+            // not.
+            return temp.substr(0, temp.rfind('\0'));
         };
 
         return get_skey(a).compare(get_skey(b));
@@ -367,8 +364,6 @@ private:
                        mtx::events::collections::TimelineEvents e,
                        const std::string &room_id);
 
-    //! Remove a room from the cache.
-    // void removeLeftRoom(lmdb::txn &txn, const std::string &room_id);
     template<class T>
     void saveStateEvents(lmdb::txn &txn,
                          lmdb::dbi &statesdb,
@@ -376,11 +371,7 @@ private:
                          lmdb::dbi &membersdb,
                          lmdb::dbi &eventsDb,
                          const std::string &room_id,
-                         const std::vector<T> &events)
-    {
-        for (const auto &e : events)
-            saveStateEvent(txn, statesdb, stateskeydb, membersdb, eventsDb, room_id, e);
-    }
+                         const std::vector<T> &events);
 
     template<class T>
     void saveStateEvent(lmdb::txn &txn,
@@ -389,162 +380,18 @@ private:
                         lmdb::dbi &membersdb,
                         lmdb::dbi &eventsDb,
                         const std::string &room_id,
-                        const T &event)
-    {
-        using namespace mtx::events;
-        using namespace mtx::events::state;
-
-        if (auto e = std::get_if<StateEvent<Member>>(&event); e != nullptr) {
-            switch (e->content.membership) {
-            //
-            // We only keep users with invite or join membership.
-            //
-            case Membership::Invite:
-            case Membership::Join: {
-                auto display_name =
-                  e->content.display_name.empty() ? e->state_key : e->content.display_name;
-
-                // Lightweight representation of a member.
-                MemberInfo tmp{display_name, e->content.avatar_url, e->content.reason};
-
-                membersdb.put(txn, e->state_key, nlohmann::json(tmp).dump());
-                break;
-            }
-            default: {
-                membersdb.del(txn, e->state_key, "");
-                break;
-            }
-            }
-
-            // BUG(Nico): Ideally we would fall through and store this in the database, but it seems
-            // to currently corrupt the db sometimes, so... let's find that bug first!
-            return;
-        } else if (std::holds_alternative<StateEvent<Encryption>>(event)) {
-            setEncryptedRoom(txn, room_id);
-            return;
-        }
-
-        std::visit(
-          [&txn, &statesdb, &stateskeydb, &eventsDb, &membersdb](const auto &e) {
-              if constexpr (isStateEvent_<decltype(e)>) {
-                  eventsDb.put(txn, e.event_id, nlohmann::json(e).dump());
-
-                  if (e.type != EventType::Unsupported) {
-                      if (std::is_same_v<std::remove_cv_t<std::remove_reference_t<decltype(e)>>,
-                                         StateEvent<mtx::events::msg::Redacted>>) {
-                          if (e.type == EventType::RoomMember)
-                              membersdb.del(txn, e.state_key, "");
-                          else if (e.state_key.empty())
-                              statesdb.del(txn, to_string(e.type));
-                          else
-                              stateskeydb.del(txn,
-                                              to_string(e.type),
-                                              nlohmann::json::object({
-                                                                       {"key", e.state_key},
-                                                                       {"id", e.event_id},
-                                                                     })
-                                                .dump());
-                      } else if (e.state_key.empty())
-                          statesdb.put(txn, to_string(e.type), nlohmann::json(e).dump());
-                      else
-                          stateskeydb.put(txn,
-                                          to_string(e.type),
-                                          nlohmann::json::object({
-                                                                   {"key", e.state_key},
-                                                                   {"id", e.event_id},
-                                                                 })
-                                            .dump());
-                  }
-              }
-          },
-          event);
-    }
+                        const T &event);
 
     template<typename T>
     std::optional<mtx::events::StateEvent<T>>
-    getStateEvent(lmdb::txn &txn, const std::string &room_id, std::string_view state_key = "")
-    {
-        try {
-            constexpr auto type = mtx::events::state_content_to_type<T>;
-            static_assert(type != mtx::events::EventType::Unsupported,
-                          "Not a supported type in state events.");
-
-            if (room_id.empty())
-                return std::nullopt;
-            const auto typeStr = to_string(type);
-
-            std::string_view value;
-            if (state_key.empty()) {
-                auto db = getStatesDb(txn, room_id);
-                if (!db.get(txn, typeStr, value)) {
-                    return std::nullopt;
-                }
-            } else {
-                auto db                   = getStatesKeyDb(txn, room_id);
-                std::string d             = nlohmann::json::object({{"key", state_key}}).dump();
-                std::string_view data     = d;
-                std::string_view typeStrV = typeStr;
-
-                auto cursor = lmdb::cursor::open(txn, db);
-                if (!cursor.get(typeStrV, data, MDB_GET_BOTH))
-                    return std::nullopt;
-
-                try {
-                    auto eventsDb = getEventsDb(txn, room_id);
-                    if (!eventsDb.get(
-                          txn, nlohmann::json::parse(data)["id"].get<std::string>(), value))
-                        return std::nullopt;
-                } catch (std::exception &) {
-                    return std::nullopt;
-                }
-            }
-
-            return nlohmann::json::parse(value).get<mtx::events::StateEvent<T>>();
-        } catch (std::exception &) {
-            return std::nullopt;
-        }
-    }
+    getStateEvent(lmdb::txn &txn, const std::string &room_id, std::string_view state_key = "");
 
     template<typename T>
     std::vector<mtx::events::StateEvent<T>>
     getStateEventsWithType(lmdb::txn &txn,
                            const std::string &room_id,
-                           mtx::events::EventType type = mtx::events::state_content_to_type<T>)
+                           mtx::events::EventType type = mtx::events::state_content_to_type<T>);
 
-    {
-        if (room_id.empty())
-            return {};
-
-        std::vector<mtx::events::StateEvent<T>> events;
-
-        {
-            auto db                   = getStatesKeyDb(txn, room_id);
-            auto eventsDb             = getEventsDb(txn, room_id);
-            const auto typeStr        = to_string(type);
-            std::string_view typeStrV = typeStr;
-            std::string_view data;
-            std::string_view value;
-
-            auto cursor = lmdb::cursor::open(txn, db);
-            bool first  = true;
-            if (cursor.get(typeStrV, data, MDB_SET)) {
-                while (cursor.get(typeStrV, data, first ? MDB_FIRST_DUP : MDB_NEXT_DUP)) {
-                    first = false;
-
-                    try {
-                        if (eventsDb.get(
-                              txn, nlohmann::json::parse(data)["id"].get<std::string>(), value))
-                            events.push_back(
-                              nlohmann::json::parse(value).get<mtx::events::StateEvent<T>>());
-                    } catch (std::exception &e) {
-                        nhlog::db()->warn("Failed to parse state event: {}", e.what());
-                    }
-                }
-            }
-        }
-
-        return events;
-    }
     void
     saveInvites(lmdb::txn &txn, const std::map<std::string, mtx::responses::InvitedRoom> &rooms);
 
@@ -568,11 +415,6 @@ private:
                       const std::set<std::string> &spaces_with_updates,
                       std::set<std::string> rooms_with_updates);
 
-    lmdb::dbi getPendingReceiptsDb(lmdb::txn &txn)
-    {
-        return lmdb::dbi::open(txn, "pending_receipts", MDB_CREATE);
-    }
-
     lmdb::dbi getEventsDb(lmdb::txn &txn, const std::string &room_id)
     {
         return lmdb::dbi::open(txn, std::string(room_id + "/events").c_str(), MDB_CREATE);
@@ -631,7 +473,7 @@ private:
     lmdb::dbi getStatesKeyDb(lmdb::txn &txn, const std::string &room_id)
     {
         auto db = lmdb::dbi::open(
-          txn, std::string(room_id + "/state_by_key").c_str(), MDB_CREATE | MDB_DUPSORT);
+          txn, std::string(room_id + "/states_key").c_str(), MDB_CREATE | MDB_DUPSORT);
         lmdb::dbi_set_dupsort(txn, db, compare_state_key);
         return db;
     }
@@ -653,16 +495,6 @@ private:
         return lmdb::dbi::open(txn, "verified", MDB_CREATE);
     }
 
-    //! Retrieves or creates the database that stores the open OLM sessions between our device
-    //! and the given curve25519 key which represents another device.
-    //!
-    //! Each entry is a map from the session_id to the pickled representation of the session.
-    lmdb::dbi getOlmSessionsDb(lmdb::txn &txn, const std::string &curve25519_key)
-    {
-        return lmdb::dbi::open(
-          txn, std::string("olm_sessions.v2/" + curve25519_key).c_str(), MDB_CREATE);
-    }
-
     QString getDisplayName(const mtx::events::StateEvent<mtx::events::state::Member> &event)
     {
         if (!event.content.display_name.empty())
@@ -692,9 +524,12 @@ private:
     lmdb::dbi inboundMegolmSessionDb_;
     lmdb::dbi outboundMegolmSessionDb_;
     lmdb::dbi megolmSessionDataDb_;
+    lmdb::dbi olmSessionDb_;
 
     lmdb::dbi encryptedRooms_;
 
+    lmdb::dbi eventExpiryBgJob_;
+
     QString localUserId_;
     QString cacheDirectory_;
 
@@ -709,3 +544,33 @@ namespace cache {
 Cache *
 client();
 }
+
+#define NHEKO_CACHE_GET_STATE_EVENT_FORWARD(Content)                                               \
+    extern template std::optional<mtx::events::StateEvent<Content>> Cache::getStateEvent(          \
+      lmdb::txn &txn, const std::string &room_id, std::string_view state_key);                     \
+                                                                                                   \
+    extern template std::vector<mtx::events::StateEvent<Content>> Cache::getStateEventsWithType(   \
+      lmdb::txn &txn, const std::string &room_id, mtx::events::EventType type);
+
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::Aliases)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::Avatar)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::CanonicalAlias)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::Create)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::Encryption)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::GuestAccess)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::HistoryVisibility)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::JoinRules)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::Member)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::Name)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::PinnedEvents)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::PowerLevels)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::Tombstone)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::ServerAcl)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::Topic)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::Widget)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::policy_rule::UserRule)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::policy_rule::RoomRule)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::policy_rule::ServerRule)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::space::Child)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::state::space::Parent)
+NHEKO_CACHE_GET_STATE_EVENT_FORWARD(mtx::events::msc2545::ImagePack)
