@@ -1,7 +1,4 @@
-// SPDX-FileCopyrightText: 2017 Konstantinos Sideris <siderisk@auth.gr>
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -9,6 +6,11 @@
 #include <QInputDialog>
 #include <QMessageBox>
 
+#include <algorithm>
+#include <unordered_set>
+
+#include <nlohmann/json.hpp>
+
 #include <mtx/responses.hpp>
 
 #include "AvatarProvider.h"
@@ -24,27 +26,20 @@
 #include "encryption/DeviceVerificationFlow.h"
 #include "encryption/Olm.h"
 #include "ui/RoomSummary.h"
-#include "ui/Theme.h"
 #include "ui/UserProfile.h"
 #include "voip/CallManager.h"
 
 #include "notifications/Manager.h"
 
+#include "timeline/RoomlistModel.h"
+#include "timeline/TimelineModel.h"
 #include "timeline/TimelineViewManager.h"
 
-#include "blurhash.hpp"
-
 ChatPage *ChatPage::instance_                    = nullptr;
 static constexpr int CHECK_CONNECTIVITY_INTERVAL = 15'000;
 static constexpr int RETRY_TIMEOUT               = 5'000;
 static constexpr size_t MAX_ONETIME_KEYS         = 50;
 
-Q_DECLARE_METATYPE(std::optional<mtx::crypto::EncryptedFile>)
-Q_DECLARE_METATYPE(std::optional<RelatedInfo>)
-Q_DECLARE_METATYPE(mtx::presence::PresenceState)
-Q_DECLARE_METATYPE(mtx::secret_storage::AesHmacSha2KeyDescription)
-Q_DECLARE_METATYPE(SecretsToDecrypt)
-
 ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
   : QObject(parent)
   , isConnected_(true)
@@ -56,12 +51,6 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
 
     instance_ = this;
 
-    qRegisterMetaType<std::optional<mtx::crypto::EncryptedFile>>();
-    qRegisterMetaType<std::optional<RelatedInfo>>();
-    qRegisterMetaType<mtx::presence::PresenceState>();
-    qRegisterMetaType<mtx::secret_storage::AesHmacSha2KeyDescription>();
-    qRegisterMetaType<SecretsToDecrypt>();
-
     view_manager_ = new TimelineViewManager(callManager_, this);
 
     connect(this,
@@ -98,6 +87,13 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
                   return;
               }
 
+              // only update spaces every 20 minutes
+              if (lastSpacesUpdate < QDateTime::currentDateTime().addSecs(-20 * 60)) {
+                  lastSpacesUpdate = QDateTime::currentDateTime();
+                  utils::updateSpaceVias();
+                  utils::removeExpiredEvents();
+              }
+
               if (!isConnected_)
                   emit connectionRestored();
           });
@@ -204,14 +200,21 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
             if (eventInDb) {
                 if (auto newRules =
                       std::get_if<mtx::events::AccountDataEvent<mtx::pushrules::GlobalRuleset>>(
-                        &*eventInDb))
+                        &*eventInDb)) {
                     pushrules =
                       std::make_unique<mtx::pushrules::PushRuleEvaluator>(newRules->content.global);
+                }
             }
         }
         if (pushrules) {
             const auto local_user = utils::localUser().toStdString();
 
+            // Desktop notifications to be sent
+            std::vector<std::tuple<QSharedPointer<TimelineModel>,
+                                   mtx::events::collections::TimelineEvents,
+                                   std::string,
+                                   std::vector<mtx::pushrules::actions::Action>>>
+              notifications;
             for (const auto &[room_id, room] : sync.rooms.join) {
                 // clear old notifications
                 for (const auto &e : room.ephemeral.events) {
@@ -267,7 +270,7 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
 
                     auto ctx = roomModel->pushrulesRoomContext();
                     std::vector<
-                      std::pair<mtx::common::Relation, mtx::events::collections::TimelineEvent>>
+                      std::pair<mtx::common::Relation, mtx::events::collections::TimelineEvents>>
                       relatedEvents;
 
                     for (const auto &event : room.timeline.events) {
@@ -283,9 +286,9 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
                         if (sender == http::client()->user_id().to_string())
                             continue;
 
-                        mtx::events::collections::TimelineEvent te{event};
-                        std::visit([room_id = room_id](auto &event_) { event_.room_id = room_id; },
-                                   te.data);
+                        mtx::events::collections::TimelineEvents te{event};
+                        std::visit(
+                          [room_id_ = room_id](auto &event_) { event_.room_id = room_id_; }, te);
 
                         if (auto encryptedEvent =
                               std::get_if<mtx::events::EncryptedEvent<mtx::events::msg::Encrypted>>(
@@ -295,23 +298,24 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
 
                             auto result = olm::decryptEvent(index, *encryptedEvent);
                             if (result.event)
-                                te.data = result.event.value();
+                                te = std::move(result.event).value();
                         }
 
                         relatedEvents.clear();
-                        for (const auto &r : mtx::accessors::relations(te.data).relations) {
+                        for (const auto &r : mtx::accessors::relations(te).relations) {
                             auto related = cache::client()->getEvent(room_id, r.event_id);
                             if (related) {
                                 relatedEvents.emplace_back(r, *related);
                                 if (auto encryptedEvent = std::get_if<
                                       mtx::events::EncryptedEvent<mtx::events::msg::Encrypted>>(
-                                      &related->data);
+                                      &related.value());
                                     encryptedEvent && userSettings_->decryptNotifications()) {
                                     MegolmSessionIndex index(room_id, encryptedEvent->content);
 
                                     auto result = olm::decryptEvent(index, *encryptedEvent);
                                     if (result.event)
-                                        relatedEvents.back().second.data = result.event.value();
+                                        relatedEvents.back().second =
+                                          std::move(result.event).value();
                                 }
                             }
                         }
@@ -330,30 +334,45 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
                                     continue;
 
                                 if (userSettings_->hasDesktopNotifications()) {
-                                    auto info = cache::singleRoomInfo(room_id);
-
-                                    AvatarProvider::resolve(
-                                      roomModel->roomAvatarUrl(),
-                                      96,
-                                      this,
-                                      [this, te, room_id = room_id, actions](QPixmap image) {
-                                          notificationsManager->postNotification(
-                                            mtx::responses::Notification{
-                                              .actions     = actions,
-                                              .event       = te.data,
-                                              .read        = false,
-                                              .profile_tag = "",
-                                              .room_id     = room_id,
-                                              .ts          = 0,
-                                            },
-                                            image.toImage());
-                                      });
+                                    notifications.emplace_back(roomModel, te, room_id, actions);
                                 }
                             }
                         }
                     }
                 }
             }
+            if (notifications.size() <= 5) {
+                for (const auto &[roomModel, te, room_id, actions] : notifications) {
+                    AvatarProvider::resolve(
+                      roomModel->roomAvatarUrl(),
+                      96,
+                      this,
+                      [this, te_ = te, room_id_ = room_id, actions_ = actions](QPixmap image) {
+                          notificationsManager->postNotification(
+                            mtx::responses::Notification{
+                              .actions     = actions_,
+                              .event       = std::move(te_),
+                              .read        = false,
+                              .profile_tag = "",
+                              .room_id     = room_id_,
+                              .ts          = 0,
+                            },
+                            image.toImage());
+                      });
+                }
+            } else if (!notifications.empty()) {
+                std::map<QSharedPointer<TimelineModel>, std::size_t> missedEvents;
+                for (const auto &[roomModel, te, room_id, actions] : notifications) {
+                    missedEvents[roomModel]++;
+                }
+                QString body;
+                for (const auto &[roomModel, nbNotifs] : missedEvents) {
+                    body += tr("%n unread message(s) in room %1\n", nullptr, nbNotifs)
+                              .arg(roomModel->roomName());
+                }
+                emit notificationsManager->systemPostNotificationCb(
+                  "", "", "New messages while away", body, QImage());
+            }
         }
     });
 
@@ -383,6 +402,26 @@ ChatPage::ChatPage(QSharedPointer<UserSettings> userSettings, QObject *parent)
       },
       Qt::QueuedConnection);
 
+    connect(
+      this,
+      &ChatPage::callFunctionOnGuiThread,
+      this,
+      [](std::function<void()> f) { f(); },
+      Qt::QueuedConnection);
+
+    connect(qobject_cast<QGuiApplication *>(QGuiApplication::instance()),
+            &QGuiApplication::focusWindowChanged,
+            this,
+            [this](QWindow *activeWindow) {
+                if (activeWindow) {
+                    nhlog::ui()->debug("Stopping inactive timer.");
+                    lastWindowActive = QDateTime();
+                } else {
+                    nhlog::ui()->debug("Starting inactive timer.");
+                    lastWindowActive = QDateTime::currentDateTime();
+                }
+            });
+
     connectCallMessage<mtx::events::voip::CallInvite>();
     connectCallMessage<mtx::events::voip::CallCandidates>();
     connectCallMessage<mtx::events::voip::CallAnswer>();
@@ -567,6 +606,10 @@ ChatPage::loadStateFromCache()
     try {
         olm::client()->load(cache::restoreOlmAccount(), cache::client()->pickleSecret());
 
+        nhlog::db()->info("Removing old cached messages");
+        cache::deleteOldData();
+        nhlog::db()->info("Message removal done");
+
         emit initializeEmptyViews();
 
         cache::calculateRoomReadStatus();
@@ -738,7 +781,26 @@ ChatPage::handleSyncResponse(const mtx::responses::Sync &res, const std::string
     nhlog::net()->debug("sync completed: {}", res.next_batch);
 
     // Ensure that we have enough one-time keys available.
-    ensureOneTimeKeyCount(res.device_one_time_keys_count, res.device_unused_fallback_key_types);
+    std::map<std::string_view, std::uint16_t> counts{res.device_one_time_keys_count.begin(),
+                                                     res.device_one_time_keys_count.end()};
+    ensureOneTimeKeyCount(counts, res.device_unused_fallback_key_types);
+
+    std::optional<mtx::events::account_data::IgnoredUsers> oldIgnoredUsers;
+    if (auto ignoreEv = std::ranges::find_if(
+          res.account_data.events,
+          [](const mtx::events::collections::RoomAccountDataEvents &e) {
+              return std::holds_alternative<
+                mtx::events::AccountDataEvent<mtx::events::account_data::IgnoredUsers>>(e);
+          });
+        ignoreEv != res.account_data.events.end()) {
+        if (auto oldEv = cache::client()->getAccountData(mtx::events::EventType::IgnoredUsers))
+            oldIgnoredUsers =
+              std::get<mtx::events::AccountDataEvent<mtx::events::account_data::IgnoredUsers>>(
+                *oldEv)
+                .content;
+        else
+            oldIgnoredUsers = mtx::events::account_data::IgnoredUsers{};
+    }
 
     // TODO: fine grained error handling
     try {
@@ -749,12 +811,34 @@ ChatPage::handleSyncResponse(const mtx::responses::Sync &res, const std::string
 
         emit syncUI(std::move(res));
 
-        // if we process a lot of syncs (1 every 200ms), this means we clean the
-        // db every 100s
-        static int syncCounter = 0;
-        if (syncCounter++ >= 500) {
-            cache::deleteOldData();
-            syncCounter = 0;
+        // if the ignored users changed, clear timeline of all affected rooms.
+        if (oldIgnoredUsers) {
+            if (auto newEv =
+                  cache::client()->getAccountData(mtx::events::EventType::IgnoredUsers)) {
+                std::vector<mtx::events::account_data::IgnoredUser> changedUsers{};
+                std::ranges::set_symmetric_difference(
+                  oldIgnoredUsers->users,
+                  std::get<mtx::events::AccountDataEvent<mtx::events::account_data::IgnoredUsers>>(
+                    *newEv)
+                    .content.users,
+                  std::back_inserter(changedUsers),
+                  {},
+                  &mtx::events::account_data::IgnoredUser::id,
+                  &mtx::events::account_data::IgnoredUser::id);
+
+                std::unordered_set<std::string> roomsToReload;
+                for (const auto &user : changedUsers) {
+                    auto commonRooms = cache::client()->getCommonRooms(user.id);
+                    for (const auto &room : commonRooms)
+                        roomsToReload.insert(room.first);
+                }
+
+                for (const auto &room : roomsToReload) {
+                    if (auto model =
+                          view_manager_->rooms()->getRoomById(QString::fromStdString(room)))
+                        model->clearTimeline();
+                }
+            }
         }
     } catch (const lmdb::map_full_error &e) {
         nhlog::db()->error("lmdb is full: {}", e.what());
@@ -763,7 +847,10 @@ ChatPage::handleSyncResponse(const mtx::responses::Sync &res, const std::string
         nhlog::db()->error("saving sync response: {}", e.what());
     }
 
-    emit trySyncCb();
+    if (shouldThrottleSync())
+        QTimer::singleShot(1000, this, &ChatPage::trySyncCb);
+    else
+        emit trySyncCb();
 }
 
 void
@@ -899,8 +986,15 @@ ChatPage::createRoom(const mtx::requests::CreateRoom &req)
         return;
     }
 
+    bool direct = req.is_direct;
+    std::string direct_user;
+    if (direct && !req.invite.empty())
+        direct_user = req.invite.front();
+
     http::client()->create_room(
-      req, [this](const mtx::responses::CreateRoom &res, mtx::http::RequestErr err) {
+      req,
+      [this, direct, direct_user](const mtx::responses::CreateRoom &res,
+                                  mtx::http::RequestErr err) {
           if (err) {
               const auto err_code = mtx::errors::to_string(err->matrix_error.errcode);
               const auto error    = err->matrix_error.error;
@@ -913,6 +1007,16 @@ ChatPage::createRoom(const mtx::requests::CreateRoom &req)
           }
 
           QString newRoomId = QString::fromStdString(res.room_id.to_string());
+
+          if (direct && !direct_user.empty()) {
+              utils::markRoomAsDirect(newRoomId,
+                                      {RoomMember{
+                                        .user_id      = QString::fromStdString(direct_user),
+                                        .display_name = "",
+                                        .avatar_url   = "",
+                                      }});
+          }
+
           emit showNotification(tr("Room %1 created.").arg(newRoomId));
           emit newRoom(newRoomId);
           emit changeToRoom(newRoomId);
@@ -1080,6 +1184,20 @@ ChatPage::setStatus(const QString &status)
       });
 }
 
+bool
+ChatPage::shouldBeUnavailable() const
+{
+    return lastWindowActive.isValid() &&
+           lastWindowActive.addSecs(60 * 5) < QDateTime::currentDateTime();
+}
+
+bool
+ChatPage::shouldThrottleSync() const
+{
+    return lastWindowActive.isValid() &&
+           lastWindowActive.addSecs(6 * 5) < QDateTime::currentDateTime();
+}
+
 mtx::presence::PresenceState
 ChatPage::currentPresence() const
 {
@@ -1090,6 +1208,12 @@ ChatPage::currentPresence() const
         return mtx::presence::unavailable;
     case UserSettings::Presence::Offline:
         return mtx::presence::offline;
+    case UserSettings::Presence::AutomaticPresence:
+        if (shouldBeUnavailable())
+            return mtx::presence::unavailable;
+        else
+            return mtx::presence::online;
+
     default:
         return mtx::presence::online;
     }
@@ -1108,7 +1232,7 @@ ChatPage::verifyOneTimeKeyCountAfterStartup()
                   return;
           }
 
-          std::map<std::string, uint16_t> key_counts;
+          std::map<std::string_view, uint16_t> key_counts;
           std::uint64_t count = 0;
           if (auto c = res.one_time_key_counts.find(mtx::crypto::SIGNED_CURVE25519);
               c == res.one_time_key_counts.end()) {
@@ -1129,7 +1253,7 @@ ChatPage::verifyOneTimeKeyCountAfterStartup()
 }
 
 void
-ChatPage::ensureOneTimeKeyCount(const std::map<std::string, uint16_t> &counts,
+ChatPage::ensureOneTimeKeyCount(const std::map<std::string_view, uint16_t> &counts,
                                 const std::optional<std::vector<std::string>> &unused_fallback_keys)
 {
     if (auto count = counts.find(mtx::crypto::SIGNED_CURVE25519); count != counts.end()) {
@@ -1248,7 +1372,7 @@ ChatPage::getBackupVersion()
                       nhlog::crypto()->info("Our backup key {} does not match the one "
                                             "used in the online backup {}",
                                             pubkey,
-                                            auth_data["public_key"]);
+                                            auth_data["public_key"].get<std::string>());
                       cache::client()->deleteBackupVersion();
                       return;
                   }
@@ -1556,7 +1680,7 @@ ChatPage::handleMatrixUri(QString uri)
 
     auto items =
       uri_.query(QUrl::ComponentFormattingOption::FullyEncoded).split('&', Qt::SkipEmptyParts);
-    for (QString item : qAsConst(items)) {
+    for (QString item : std::as_const(items)) {
         nhlog::ui()->info("item: {}", item.toStdString());
 
         if (item.startsWith(QLatin1String("action="))) {
@@ -1664,7 +1788,9 @@ ChatPage::isRoomActive(const QString &room_id)
 void
 ChatPage::removeAllNotifications()
 {
-#if defined(Q_OS_LINUX)
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
     notificationsManager->closeAllNotifications();
 #endif
 }
+
+#include "moc_ChatPage.cpp"
