@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -11,7 +9,6 @@
 
 #include "CompletionModelRoles.h"
 #include "Logging.h"
-#include "Utils.h"
 
 CompletionProxyModel::CompletionProxyModel(QAbstractItemModel *model,
                                            int max_mistakes,
@@ -31,7 +28,7 @@ CompletionProxyModel::CompletionProxyModel(QAbstractItemModel *model,
             finder.toNextBoundary();
             auto end = finder.position();
 
-            auto ref = str.midRef(start, end - start).trimmed();
+            auto ref = QStringView(str).mid(start, end - start).trimmed();
             if (!ref.isEmpty())
                 trie_.insert<ElementRank::second>(ref.toUcs4(), id);
         } while (finder.position() < str.size());
@@ -47,7 +44,7 @@ CompletionProxyModel::CompletionProxyModel(QAbstractItemModel *model,
         auto string1 = sourceModel()
                          ->data(sourceModel()->index(i, 0), CompletionModel::SearchRole)
                          .toString()
-                         .toLower();
+                         .toCaseFolded();
         if (!string1.isEmpty()) {
             trie_.insert<ElementRank::first>(string1.toUcs4(), i);
             insertParts(string1, i);
@@ -56,7 +53,7 @@ CompletionProxyModel::CompletionProxyModel(QAbstractItemModel *model,
         auto string2 = sourceModel()
                          ->data(sourceModel()->index(i, 0), CompletionModel::SearchRole2)
                          .toString()
-                         .toLower();
+                         .toCaseFolded();
         if (!string2.isEmpty()) {
             trie_.insert<ElementRank::first>(string2.toUcs4(), i);
             insertParts(string2, i);
@@ -76,7 +73,7 @@ CompletionProxyModel::CompletionProxyModel(QAbstractItemModel *model,
       &CompletionProxyModel::newSearchString,
       this,
       [this](const QString &s) {
-          searchString_ = s.toLower();
+          searchString_ = s.toCaseFolded();
           invalidate();
       },
       Qt::QueuedConnection);
@@ -172,3 +169,5 @@ CompletionProxyModel::setSearchString(const QString &s)
 {
     emit newSearchString(s);
 }
+
+#include "moc_CompletionProxyModel.cpp"
