@@ -1,20 +1,14 @@
-// SPDX-FileCopyrightText: 2017 Konstantinos Sideris <siderisk@auth.gr>
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
 #include <QObject>
+#include <QQmlEngine>
 #include <QVariantList>
 
-namespace mtx {
-namespace responses {
-struct Login;
-}
-}
+#include <mtx/responses/login.hpp>
 
 struct SSOProvider
 {
@@ -36,6 +30,7 @@ public:
 class LoginPage : public QObject
 {
     Q_OBJECT
+    QML_NAMED_ELEMENT(Login)
 
     Q_PROPERTY(QString mxid READ mxid WRITE setMxid NOTIFY matrixIdChanged)
     Q_PROPERTY(QString homeserver READ homeserver WRITE setHomeserver NOTIFY homeserverChanged)
@@ -100,6 +95,8 @@ public:
         return "Nheko on Windows";
 #elif defined(Q_OS_FREEBSD)
         return "Nheko on FreeBSD";
+#elif defined(Q_OS_OPENBSD)
+        return "Nheko on OpenBSD";
 #else
         return "Nheko";
 #endif
