@@ -1,5 +1,4 @@
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -84,7 +83,7 @@ std::map<std::string, mtx::events::state::power_level_t, std::less<>>
 PowerlevelsTypeListModel::toEvents() const
 {
     std::map<std::string, mtx::events::state::power_level_t, std::less<>> m;
-    for (const auto &[key, pl] : qAsConst(types))
+    for (const auto &[key, pl] : std::as_const(types))
         if (key.find('.') != std::string::npos)
             m[key] = pl;
     return m;
@@ -92,7 +91,7 @@ PowerlevelsTypeListModel::toEvents() const
 mtx::events::state::power_level_t
 PowerlevelsTypeListModel::kick() const
 {
-    for (const auto &[key, pl] : qAsConst(types))
+    for (const auto &[key, pl] : std::as_const(types))
         if (key == "kick")
             return pl;
     return powerLevels_.users_default;
@@ -100,7 +99,7 @@ PowerlevelsTypeListModel::kick() const
 mtx::events::state::power_level_t
 PowerlevelsTypeListModel::invite() const
 {
-    for (const auto &[key, pl] : qAsConst(types))
+    for (const auto &[key, pl] : std::as_const(types))
         if (key == "invite")
             return pl;
     return powerLevels_.users_default;
@@ -108,7 +107,7 @@ PowerlevelsTypeListModel::invite() const
 mtx::events::state::power_level_t
 PowerlevelsTypeListModel::ban() const
 {
-    for (const auto &[key, pl] : qAsConst(types))
+    for (const auto &[key, pl] : std::as_const(types))
         if (key == "ban")
             return pl;
     return powerLevels_.users_default;
@@ -116,7 +115,7 @@ PowerlevelsTypeListModel::ban() const
 mtx::events::state::power_level_t
 PowerlevelsTypeListModel::eventsDefault() const
 {
-    for (const auto &[key, pl] : qAsConst(types))
+    for (const auto &[key, pl] : std::as_const(types))
         if (key == "zdefault_events")
             return pl;
     return powerLevels_.users_default;
@@ -124,7 +123,7 @@ PowerlevelsTypeListModel::eventsDefault() const
 mtx::events::state::power_level_t
 PowerlevelsTypeListModel::stateDefault() const
 {
-    for (const auto &[key, pl] : qAsConst(types))
+    for (const auto &[key, pl] : std::as_const(types))
         if (key == "zdefault_states")
             return pl;
     return powerLevels_.users_default;
@@ -400,7 +399,7 @@ std::map<std::string, mtx::events::state::power_level_t, std::less<>>
 PowerlevelsUserListModel::toUsers() const
 {
     std::map<std::string, mtx::events::state::power_level_t, std::less<>> m;
-    for (const auto &[key, pl] : qAsConst(users))
+    for (const auto &[key, pl] : std::as_const(users))
         if (key.size() > 0 && key.at(0) == '@')
             m[key] = pl;
     return m;
@@ -408,7 +407,7 @@ PowerlevelsUserListModel::toUsers() const
 mtx::events::state::power_level_t
 PowerlevelsUserListModel::usersDefault() const
 {
-    for (const auto &[key, pl] : qAsConst(users))
+    for (const auto &[key, pl] : std::as_const(users))
         if (key == "default")
             return pl;
     return powerLevels_.users_default;
@@ -636,7 +635,7 @@ PowerlevelEditingModels::updateSpacesModel()
 void
 PowerlevelEditingModels::addRole(int pl)
 {
-    for (const auto &e : qAsConst(types_.types))
+    for (const auto &e : std::as_const(types_.types))
         if (pl == int(e.pl))
             return;
 
@@ -728,9 +727,12 @@ struct PowerLevelApplier
           [self = *this](const mtx::responses::EventId &, mtx::http::RequestErr e) mutable {
               if (e) {
                   if (e->status_code == 429 && e->matrix_error.retry_after.count() != 0) {
-                      QTimer::singleShot(e->matrix_error.retry_after,
-                                         ChatPage::instance(),
-                                         [self = std::move(self)]() mutable { self.next(); });
+                      ChatPage::instance()->callFunctionOnGuiThread(
+                        [self = std::move(self), interval = e->matrix_error.retry_after]() {
+                            QTimer::singleShot(interval,
+                                               ChatPage::instance(),
+                                               [self = std::move(self)]() mutable { self.next(); });
+                        });
                       return;
                   }
 
@@ -750,7 +752,7 @@ PowerlevelsSpacesListModel::commit()
 {
     std::vector<std::string> spacesToApplyTo;
 
-    for (const auto &s : qAsConst(spaces))
+    for (const auto &s : std::as_const(spaces))
         if (s.apply)
             spacesToApplyTo.push_back(s.roomid);
 
@@ -825,3 +827,5 @@ PowerlevelsSpacesListModel::roleNames() const
       {ApplyPermissions, "applyPermissions"},
     };
 }
+
+#include "moc_PowerlevelsEditModels.cpp"
