@@ -1,17 +1,16 @@
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
 #include <QAbstractListModel>
+#include <QQmlEngine>
 #include <QSortFilterProxyModel>
 
+#include <mtx/events/event_type.hpp>
 #include <mtx/events/power_levels.hpp>
 
-#include "CacheStructs.h"
-
 class PowerlevelsTypeListModel final : public QAbstractListModel
 {
     Q_OBJECT
@@ -197,6 +196,9 @@ class PowerlevelEditingModels final : public QObject
 {
     Q_OBJECT
 
+    QML_ELEMENT
+    QML_UNCREATABLE("Please use editPowerlevels to create the models")
+
     Q_PROPERTY(PowerlevelsUserListModel *users READ users CONSTANT)
     Q_PROPERTY(PowerlevelsTypeListModel *types READ types CONSTANT)
     Q_PROPERTY(PowerlevelsSpacesListModel *spaces READ spaces CONSTANT)
