@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -8,7 +6,6 @@
 
 #include <QUrl>
 
-#include "Cache.h"
 #include "Cache_p.h"
 #include "CompletionModelRoles.h"
 #include "UserSettingsPage.h"
@@ -22,18 +19,25 @@ RoomsModel::RoomsModel(bool showOnlyRoomWithAliases, QObject *parent)
 
     if (showOnlyRoomWithAliases_)
         utils::erase_if(rooms, [](auto &r) { return r.alias.empty(); });
+
+    std::ranges::sort(rooms,
+                      [](auto &a, auto &b) { return a.recent_activity > b.recent_activity; });
 }
 
 QHash<int, QByteArray>
 RoomsModel::roleNames() const
 {
-    return {{CompletionModel::CompletionRole, "completionRole"},
-            {CompletionModel::SearchRole, "searchRole"},
-            {CompletionModel::SearchRole2, "searchRole2"},
-            {Roles::RoomAlias, "roomAlias"},
-            {Roles::AvatarUrl, "avatarUrl"},
-            {Roles::RoomID, "roomid"},
-            {Roles::RoomName, "roomName"}};
+    return {
+      {CompletionModel::CompletionRole, "completionRole"},
+      {CompletionModel::SearchRole, "searchRole"},
+      {CompletionModel::SearchRole2, "searchRole2"},
+      {Roles::RoomAlias, "roomAlias"},
+      {Roles::AvatarUrl, "avatarUrl"},
+      {Roles::RoomID, "roomid"},
+      {Roles::RoomName, "roomName"},
+      {Roles::IsTombstoned, "isTombstoned"},
+      {Roles::IsSpace, "isSpace"},
+    };
 }
 
 QVariant
@@ -64,6 +68,10 @@ RoomsModel::data(const QModelIndex &index, int role) const
               cache::client()->singleRoomInfo(rooms[index.row()].id).avatar_url);
         case Roles::RoomID:
             return QString::fromStdString(rooms[index.row()].id).toHtmlEscaped();
+        case Roles::IsTombstoned:
+            return rooms[index.row()].is_tombstoned;
+        case Roles::IsSpace:
+            return rooms[index.row()].is_space;
         }
     }
     return {};
