@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -8,6 +6,7 @@
 
 #include <QAbstractListModel>
 #include <QList>
+#include <QQmlEngine>
 #include <QUrl>
 
 #include <mtx/events/mscs/image_packs.hpp>
@@ -17,6 +16,8 @@
 class SingleImagePackModel final : public QAbstractListModel
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_UNCREATABLE("")
 
     Q_PROPERTY(QString roomid READ roomid CONSTANT)
     Q_PROPERTY(bool fromSpace READ fromSpace CONSTANT)
@@ -69,10 +70,13 @@ public:
     void setIsEmotePack(bool val);
 
     Q_INVOKABLE void save();
+    Q_INVOKABLE void remove();
     Q_INVOKABLE void addStickers(QList<QUrl> files);
     Q_INVOKABLE void remove(int index);
     Q_INVOKABLE void setAvatar(QUrl file);
 
+    static std::string unconflictingStatekey(const std::string &roomid, const std::string &key);
+
 signals:
     void globallyEnabledChanged();
     void statekeyChanged();
@@ -89,6 +93,8 @@ private slots:
     void addImageCb(std::string uri, std::string filename, mtx::common::ImageInfo info);
 
 private:
+    std::string unconflictingShortcode(const std::string &shortcode);
+
     std::string roomid_;
     std::string statekey_, old_statekey_;
 
