@@ -1,7 +1,4 @@
-// SPDX-FileCopyrightText: 2017 Konstantinos Sideris <siderisk@auth.gr>
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -9,23 +6,17 @@
 
 #include <QAbstractListModel>
 #include <QProcessEnvironment>
+#include <QQmlEngine>
 #include <QSettings>
 #include <QSharedPointer>
 
 #include <optional>
 
-class Toggle;
-class QLabel;
-class QFormLayout;
-class QComboBox;
-class QFontComboBox;
-class QSpinBox;
-class QHBoxLayout;
-class QVBoxLayout;
-
 class UserSettings final : public QObject
 {
     Q_OBJECT
+    QML_NAMED_ELEMENT(Settings)
+    QML_SINGLETON
 
     Q_PROPERTY(QString theme READ theme WRITE setTheme NOTIFY themeChanged)
     Q_PROPERTY(bool messageHoverHighlight READ messageHoverHighlight WRITE setMessageHoverHighlight
@@ -47,7 +38,9 @@ class UserSettings final : public QObject
     Q_PROPERTY(bool typingNotifications READ typingNotifications WRITE setTypingNotifications NOTIFY
                  typingNotificationsChanged)
     Q_PROPERTY(bool sortByImportance READ sortByImportance WRITE setSortByImportance NOTIFY
-                 roomSortingChanged)
+                 roomSortingChangedImportance)
+    Q_PROPERTY(bool sortByAlphabet READ sortByAlphabet WRITE setSortByAlphabet NOTIFY
+                 roomSortingChangedAlphabetical)
     Q_PROPERTY(bool buttonsInTimeline READ buttonsInTimeline WRITE setButtonsInTimeline NOTIFY
                  buttonInTimelineChanged)
     Q_PROPERTY(bool readReceipts READ readReceipts WRITE setReadReceipts NOTIFY readReceiptsChanged)
@@ -77,6 +70,7 @@ class UserSettings final : public QObject
     Q_PROPERTY(int communityListWidth READ communityListWidth WRITE setCommunityListWidth NOTIFY
                  communityListWidthChanged)
     Q_PROPERTY(bool mobileMode READ mobileMode WRITE setMobileMode NOTIFY mobileModeChanged)
+    Q_PROPERTY(bool disableSwipe READ disableSwipe WRITE setDisableSwipe NOTIFY disableSwipeChanged)
     Q_PROPERTY(double fontSize READ fontSize WRITE setFontSize NOTIFY fontSizeChanged)
     Q_PROPERTY(QString font READ font WRITE setFontFamily NOTIFY fontChanged)
     Q_PROPERTY(QString emojiFont READ emojiFont WRITE setEmojiFontFamily NOTIFY emojiFontChanged)
@@ -124,12 +118,33 @@ class UserSettings final : public QObject
                  hiddenWidgetsChanged)
     Q_PROPERTY(
       bool exposeDBusApi READ exposeDBusApi WRITE setExposeDBusApi NOTIFY exposeDBusApiChanged)
+    Q_PROPERTY(bool updateSpaceVias READ updateSpaceVias WRITE setUpdateSpaceVias NOTIFY
+                 updateSpaceViasChanged)
+    Q_PROPERTY(bool expireEvents READ expireEvents WRITE setExpireEvents NOTIFY expireEventsChanged)
 
     UserSettings();
 
 public:
     static QSharedPointer<UserSettings> instance();
     static void initialize(std::optional<QString> profile);
+    static UserSettings *create(QQmlEngine *qmlEngine, QJSEngine *)
+    {
+        // The instance has to exist before it is used. We cannot replace it.
+        Q_ASSERT(instance());
+
+        // The engine has to have the same thread affinity as the singleton.
+        Q_ASSERT(qmlEngine->thread() == instance()->thread());
+
+        // There can only be one engine accessing the singleton.
+        static QJSEngine *s_engine = nullptr;
+        if (s_engine)
+            Q_ASSERT(qmlEngine == s_engine);
+        else
+            s_engine = qmlEngine;
+
+        QJSEngine::setObjectOwnership(instance().get(), QJSEngine::CppOwnership);
+        return instance().get();
+    }
 
     QSettings *qsettings() { return &settings; }
 
@@ -151,6 +166,7 @@ public:
     void setTray(bool state);
     void setStartInTray(bool state);
     void setMobileMode(bool mode);
+    void setDisableSwipe(bool mode);
     void setFontSize(double size);
     void setFontFamily(QString family);
     void setEmojiFontFamily(QString family);
@@ -164,6 +180,7 @@ public:
     void setReadReceipts(bool state);
     void setTypingNotifications(bool state);
     void setSortByImportance(bool state);
+    void setSortByAlphabet(bool state);
     void setButtonsInTimeline(bool state);
     void setTimelineMaxWidth(int state);
     void setCommunityListWidth(int state);
@@ -196,6 +213,7 @@ public:
     void setUserId(QString userId);
     void setAccessToken(QString accessToken);
     void setDeviceId(QString deviceId);
+    void setCurrentTagId(QString currentTagId);
     void setHomeserver(QString homeserver);
     void setDisableCertificateValidation(bool disabled);
     void setHiddenTags(const QStringList &hiddenTags);
@@ -208,6 +226,8 @@ public:
     void setOpenVideoExternal(bool state);
     void setCollapsedSpaces(QList<QStringList> spaces);
     void setExposeDBusApi(bool state);
+    void setUpdateSpaceVias(bool state);
+    void setExpireEvents(bool state);
 
     QString theme() const { return !theme_.isEmpty() ? theme_ : defaultTheme_; }
     bool messageHoverHighlight() const { return messageHoverHighlight_; }
@@ -231,8 +251,10 @@ public:
     bool animateImagesOnHover() const { return animateImagesOnHover_; }
     bool typingNotifications() const { return typingNotifications_; }
     bool sortByImportance() const { return sortByImportance_; }
+    bool sortByAlphabet() const { return sortByAlphabet_; }
     bool buttonsInTimeline() const { return buttonsInTimeline_; }
     bool mobileMode() const { return mobileMode_; }
+    bool disableSwipe() const { return disableSwipe_; }
     bool readReceipts() const { return readReceipts_; }
     bool hasDesktopNotifications() const { return hasDesktopNotifications_; }
     bool hasAlertOnNotification() const { return hasAlertOnNotification_; }
@@ -268,6 +290,7 @@ public:
     QString userId() const { return userId_; }
     QString accessToken() const { return accessToken_; }
     QString deviceId() const { return deviceId_; }
+    QString currentTagId() const { return currentTagId_; }
     QString homeserver() const { return homeserver_; }
     bool disableCertificateValidation() const { return disableCertificateValidation_; }
     QStringList hiddenTags() const { return hiddenTags_; }
@@ -280,11 +303,14 @@ public:
     bool openVideoExternal() const { return openVideoExternal_; }
     QList<QStringList> collapsedSpaces() const { return collapsedSpaces_; }
     bool exposeDBusApi() const { return exposeDBusApi_; }
+    bool updateSpaceVias() const { return updateSpaceVias_; }
+    bool expireEvents() const { return expireEvents_; }
 
 signals:
     void groupViewStateChanged(bool state);
     void scrollbarsInRoomlistChanged(bool state);
-    void roomSortingChanged(bool state);
+    void roomSortingChangedImportance(bool state);
+    void roomSortingChangedAlphabetical(bool state);
     void themeChanged(QString state);
     void messageHoverHighlightChanged(bool state);
     void enlargeEmojiOnlyMessagesChanged(bool state);
@@ -312,6 +338,7 @@ signals:
     void roomListWidthChanged(int state);
     void communityListWidthChanged(int state);
     void mobileModeChanged(bool mode);
+    void disableSwipeChanged(bool state);
     void fontSizeChanged(double state);
     void fontChanged(QString state);
     void emojiFontChanged(QString state);
@@ -342,6 +369,8 @@ signals:
     void hiddenWidgetsChanged();
     void recentReactionsChanged();
     void exposeDBusApiChanged(bool state);
+    void updateSpaceViasChanged(bool state);
+    void expireEventsChanged(bool state);
 
 private:
     // Default to system theme if QT_QPA_PLATFORMTHEME var is set.
@@ -364,6 +393,7 @@ private:
     bool animateImagesOnHover_;
     bool typingNotifications_;
     bool sortByImportance_;
+    bool sortByAlphabet_;
     bool buttonsInTimeline_;
     bool readReceipts_;
     bool hasDesktopNotifications_;
@@ -380,6 +410,7 @@ private:
     bool onlyShareKeysWithVerifiedUsers_;
     bool useOnlineKeyBackup_;
     bool mobileMode_;
+    bool disableSwipe_;
     int timelineMaxWidth_;
     int roomListWidth_;
     int communityListWidth_;
@@ -402,6 +433,7 @@ private:
     QString userId_;
     QString accessToken_;
     QString deviceId_;
+    QString currentTagId_;
     QString homeserver_;
     QStringList hiddenTags_;
     QStringList mutedTags_;
@@ -413,22 +445,27 @@ private:
     bool openImageExternal_;
     bool openVideoExternal_;
     bool exposeDBusApi_;
+    bool updateSpaceVias_;
+    bool expireEvents_;
 
     QSettings settings;
 
     static QSharedPointer<UserSettings> instance_;
 };
 
-class UserSettingsModel final : public QAbstractListModel
+class UserSettingsModel : public QAbstractListModel
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_SINGLETON
 
     enum Indices
     {
         GeneralSection,
         Theme,
         MobileMode,
-#ifndef Q_OS_MAC
+        DisableSwipe,
+#ifndef Q_OS_MACOS
         ScaleFactor,
 #endif
         Font,
@@ -442,6 +479,7 @@ class UserSettingsModel final : public QAbstractListModel
 #ifdef NHEKO_DBUS_SYS
         ExposeDBusApi,
 #endif
+        UpdateSpaceVias,
 
         AccessibilitySection,
         ReducedMotion,
@@ -465,6 +503,7 @@ class UserSettingsModel final : public QAbstractListModel
         SidebarSection,
         GroupView,
         SortByImportance,
+        SortByAlphabet,
         DecryptSidebar,
         SpaceNotifications,
 
@@ -472,6 +511,11 @@ class UserSettingsModel final : public QAbstractListModel
         Tray,
         StartInTray,
 
+        MessageVisibilitySection,
+        ExpireEvents,
+        HiddenTimelineEvents,
+        IgnoredUsers,
+
         NotificationsSection,
         DesktopNotifications,
         AlertOnNotification,
@@ -507,7 +551,7 @@ class UserSettingsModel final : public QAbstractListModel
         COUNT,
         // hidden for now
         AccessToken,
-#ifdef Q_OS_MAC
+#ifdef Q_OS_MACOS
         ScaleFactor,
 #endif
 #ifndef NHEKO_DBUS_SYS
@@ -528,6 +572,8 @@ public:
         KeyStatus,
         SessionKeyImportExport,
         XSignKeysRequestDownload,
+        ConfigureHiddenEvents,
+        ManageIgnoredUsers,
     };
     Q_ENUM(Types);
 
