@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -24,7 +22,6 @@
 #include "Logging.h"
 #include "MatrixClient.h"
 #include "UserSettingsPage.h"
-#include "Utils.h"
 
 namespace {
 auto client_ = std::make_unique<mtx::crypto::OlmClient>();
@@ -632,6 +629,7 @@ encrypt_group_message(const std::string &room_id, const std::string &device_id,
         // Saving the new megolm session.
         GroupSessionData session_data{};
         session_data.message_index              = 0;
+        session_data.trusted                    = true;
         session_data.timestamp                  = QDateTime::currentMSecsSinceEpoch();
         session_data.sender_claimed_ed25519_key = olm::client()->identity_keys().ed25519;
         session_data.sender_key                 = olm::client()->identity_keys().curve25519;
@@ -721,7 +719,7 @@ try_olm_decryption(const std::string &sender_key, const mtx::events::msg::OlmCip
             nhlog::crypto()->debug("Updated olm session: {}",
                                    mtx::crypto::session_id(session->get()));
             cache::saveOlmSession(
-              id, std::move(session.value()), QDateTime::currentMSecsSinceEpoch());
+              sender_key, std::move(session.value()), QDateTime::currentMSecsSinceEpoch());
         } catch (const mtx::crypto::olm_exception &e) {
             nhlog::crypto()->debug("failed to decrypt olm message ({}, {}) with {}: {}",
                                    msg.type,
@@ -756,13 +754,16 @@ create_inbound_megolm_session(const mtx::events::DeviceEvent<mtx::events::msg::R
     index.session_id = roomKey.content.session_id;
 
     try {
+        auto megolm_session =
+          olm::client()->init_inbound_group_session(roomKey.content.session_key);
+
         GroupSessionData data{};
         data.forwarding_curve25519_key_chain = {sender_key};
         data.sender_claimed_ed25519_key      = sender_ed25519;
         data.sender_key                      = sender_key;
 
-        auto megolm_session =
-          olm::client()->init_inbound_group_session(roomKey.content.session_key);
+        data.trusted = olm_inbound_group_session_is_verified(megolm_session.get());
+
         backup_session_key(index, data, megolm_session);
         cache::saveInboundMegolmSession(index, std::move(megolm_session), data);
     } catch (const lmdb::error &e) {
@@ -795,14 +796,9 @@ import_inbound_megolm_session(
         data.forwarding_curve25519_key_chain = roomKey.content.forwarding_curve25519_key_chain;
         data.sender_claimed_ed25519_key      = roomKey.content.sender_claimed_ed25519_key;
         data.sender_key                      = roomKey.content.sender_key;
-        // may have come from online key backup, so we can't trust it...
-        data.trusted = false;
-        // if we got it forwarded from the sender, assume it is trusted. They may still have
-        // used key backup, but it is unlikely.
-        if (roomKey.content.forwarding_curve25519_key_chain.size() == 1 &&
-            roomKey.content.forwarding_curve25519_key_chain.back() == roomKey.content.sender_key) {
-            data.trusted = true;
-        }
+        // Keys from online key backup won't have a signature, so they will be untrusted. But the
+        // original sender might send us a signed session.
+        data.trusted = olm_inbound_group_session_is_verified(megolm_session.get());
 
         backup_session_key(index, data, megolm_session);
         cache::saveInboundMegolmSession(index, std::move(megolm_session), data);
@@ -1026,6 +1022,7 @@ lookup_keybackup(const std::string &room, const std::string &session_id)
               data.forwarding_curve25519_key_chain = session.forwarding_curve25519_key_chain;
               data.sender_claimed_ed25519_key      = session.sender_claimed_keys["ed25519"];
               data.sender_key                      = session.sender_key;
+
               // online key backup can't be trusted, because anyone can upload to it.
               data.trusted = false;
 
@@ -1262,7 +1259,7 @@ decryptEvent(const MegolmSessionIndex &index,
     } catch (const lmdb::error &e) {
         return {DecryptionErrorCode::DbError, e.what(), std::nullopt};
     } catch (const mtx::crypto::olm_exception &e) {
-        if (e.error_code() == mtx::crypto::OlmErrorCode::UNKNOWN_MESSAGE_INDEX)
+        if (e.error_code() == mtx::crypto::OlmErrorCode::OLM_UNKNOWN_MESSAGE_INDEX)
             return {DecryptionErrorCode::MissingSessionIndex, e.what(), std::nullopt};
         return {DecryptionErrorCode::DecryptionFailed, e.what(), std::nullopt};
     }
@@ -1275,28 +1272,46 @@ decryptEvent(const MegolmSessionIndex &index,
         body["origin_server_ts"] = event.origin_server_ts;
         body["unsigned"]         = event.unsigned_data;
 
-        mtx::events::collections::TimelineEvent te;
-        from_json(body, te);
+        mtx::events::collections::TimelineEvents te =
+          body.get<mtx::events::collections::TimelineEvents>();
 
         // relations are unencrypted in content...
-        mtx::accessors::set_relations(te.data, std::move(event.content.relations));
+        mtx::accessors::set_relations(te, std::move(event.content.relations));
 
-        return {DecryptionErrorCode::NoError, std::nullopt, std::move(te.data)};
+        return {DecryptionErrorCode::NoError, std::nullopt, std::move(te)};
     } catch (std::exception &e) {
         return {DecryptionErrorCode::ParsingFailed, e.what(), std::nullopt};
     }
 }
 
 crypto::Trust
-calculate_trust(const std::string &user_id, const MegolmSessionIndex &index)
+calculate_trust(const std::string &user_id,
+                const std::string &room_id,
+                const mtx::events::msg::Encrypted &event)
 {
-    auto status              = cache::client()->verificationStatus(user_id);
+    auto index               = MegolmSessionIndex(room_id, event);
     auto megolmData          = cache::client()->getMegolmSessionData(index);
-    crypto::Trust trustlevel = crypto::Trust::Unverified;
+    crypto::Trust trustlevel = crypto::Trust::MessageUnverified;
+
+    try {
+        auto session = cache::client()->getInboundMegolmSession(index);
+        if (!session) {
+            return trustlevel;
+        }
+
+        olm::client()->decrypt_group_message(session.get(), event.ciphertext);
+    } catch (const lmdb::error &e) {
+        return trustlevel;
+    } catch (const mtx::crypto::olm_exception &e) {
+        return trustlevel;
+    }
+
+    auto status = cache::client()->verificationStatus(user_id);
 
     if (megolmData && megolmData->trusted &&
-        status.verified_device_keys.count(megolmData->sender_key))
+        status.verified_device_keys.count(megolmData->sender_key)) {
         trustlevel = status.verified_device_keys.at(megolmData->sender_key);
+    }
 
     return trustlevel;
 }
@@ -1308,7 +1323,7 @@ send_encrypted_to_device_messages(const std::map<std::string, std::vector<std::s
                                   const mtx::events::collections::DeviceEvents &event,
                                   bool force_new_session)
 {
-    static QMap<QPair<std::string, std::string>, qint64> rateLimit;
+    static QMap<std::pair<std::string, std::string>, qint64> rateLimit;
 
     nlohmann::json ev_json = std::visit([](const auto &e) { return nlohmann::json(e); }, event);
 
@@ -1365,12 +1380,12 @@ send_encrypted_to_device_messages(const std::map<std::string, std::vector<std::s
 
                 auto session = cache::getLatestOlmSession(device_curve);
                 if (!session || force_new_session) {
-                    if (rateLimit.value(QPair(user, device)) + 60 * 60 * 10 < currentTime) {
+                    if (rateLimit.value(std::pair(user, device)) + 60 * 60 * 10 < currentTime) {
                         claims.one_time_keys[user][device] = mtx::crypto::SIGNED_CURVE25519;
                         pks[user][device].ed25519          = d.keys.at("ed25519:" + device);
                         pks[user][device].curve25519       = d.keys.at("curve25519:" + device);
 
-                        rateLimit.insert(QPair(user, device), currentTime);
+                        rateLimit.insert(std::pair(user, device), currentTime);
                     } else {
                         nhlog::crypto()->warn("Not creating new session with {}:{} "
                                               "because of rate limit",
@@ -1571,13 +1586,13 @@ send_encrypted_to_device_messages(const std::map<std::string, std::vector<std::s
                       }
 
                       auto currentTime = QDateTime::currentSecsSinceEpoch();
-                      if (rateLimit.value(QPair(user.first, device_id.get())) + 60 * 60 * 10 <
+                      if (rateLimit.value(std::pair(user.first, device_id.get())) + 60 * 60 * 10 <
                           currentTime) {
                           deviceKeys[user_id].emplace(device_id, pks);
                           claim_keys.one_time_keys[user.first][device_id] =
                             mtx::crypto::SIGNED_CURVE25519;
 
-                          rateLimit.insert(QPair(user.first, device_id.get()), currentTime);
+                          rateLimit.insert(std::pair(user.first, device_id.get()), currentTime);
                       } else {
                           nhlog::crypto()->warn("Not creating new session with {}:{} "
                                                 "because of rate limit",
@@ -1668,10 +1683,10 @@ request_cross_signing_keys()
         });
     };
 
-    request(mtx::secret_storage::secrets::cross_signing_master);
-    request(mtx::secret_storage::secrets::cross_signing_self_signing);
-    request(mtx::secret_storage::secrets::cross_signing_user_signing);
-    request(mtx::secret_storage::secrets::megolm_backup_v1);
+    request(std::string(mtx::secret_storage::secrets::cross_signing_master));
+    request(std::string(mtx::secret_storage::secrets::cross_signing_self_signing));
+    request(std::string(mtx::secret_storage::secrets::cross_signing_user_signing));
+    request(std::string(mtx::secret_storage::secrets::megolm_backup_v1));
 }
 
 namespace {
@@ -1728,22 +1743,22 @@ download_cross_signing_keys()
 
                             if (backup_key && !backup_key->encrypted.empty())
                                 secrets[backup_key->encrypted.begin()->first]
-                                       [secrets::megolm_backup_v1] =
+                                       [std::string(secrets::megolm_backup_v1)] =
                                          backup_key->encrypted.begin()->second;
 
                             if (master_key && !master_key->encrypted.empty())
                                 secrets[master_key->encrypted.begin()->first]
-                                       [secrets::cross_signing_master] =
+                                       [std::string(secrets::cross_signing_master)] =
                                          master_key->encrypted.begin()->second;
 
                             if (self_signing_key && !self_signing_key->encrypted.empty())
                                 secrets[self_signing_key->encrypted.begin()->first]
-                                       [secrets::cross_signing_self_signing] =
+                                       [std::string(secrets::cross_signing_self_signing)] =
                                          self_signing_key->encrypted.begin()->second;
 
                             if (user_signing_key && !user_signing_key->encrypted.empty())
                                 secrets[user_signing_key->encrypted.begin()->first]
-                                       [secrets::cross_signing_user_signing] =
+                                       [std::string(secrets::cross_signing_user_signing)] =
                                          user_signing_key->encrypted.begin()->second;
 
                             for (const auto &[key, secret_] : secrets)
@@ -1755,3 +1770,5 @@ download_cross_signing_keys()
 }
 
 } // namespace olm
+
+#include "moc_Olm.cpp"
