@@ -1,7 +1,4 @@
-// SPDX-FileCopyrightText: 2012 Roland Hieber <rohieb@rohieb.name>
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -25,6 +22,7 @@
 
 #include "Cache.h"
 #include "EventAccessors.h"
+#include "Logging.h"
 #include "MxcImageProvider.h"
 #include "UserSettingsPage.h"
 #include "Utils.h"
@@ -39,14 +37,14 @@ NotificationsManager::NotificationsManager(QObject *parent)
          this)
   , hasMarkup_{std::invoke([this]() -> bool {
       auto caps = dbus.call("GetCapabilities").arguments();
-      for (const auto &x : qAsConst(caps))
+      for (const auto &x : std::as_const(caps))
           if (x.toStringList().contains("body-markup"))
               return true;
       return false;
   })}
   , hasImages_{std::invoke([this]() -> bool {
       auto caps = dbus.call("GetCapabilities").arguments();
-      for (const auto &x : qAsConst(caps))
+      for (const auto &x : std::as_const(caps))
           if (x.toStringList().contains("body-images"))
               return true;
       return false;
@@ -61,6 +59,12 @@ NotificationsManager::NotificationsManager(QObject *parent)
                                           QStringLiteral("ActionInvoked"),
                                           this,
                                           SLOT(actionInvoked(uint,QString)));
+    QDBusConnection::sessionBus().connect(QStringLiteral("org.freedesktop.Notifications"),
+                                          QStringLiteral("/org/freedesktop/Notifications"),
+                                          QStringLiteral("org.freedesktop.Notifications"),
+                                          QStringLiteral("ActivationToken"),
+                                          this,
+                                          SLOT(activationToken(uint,QString)));
     QDBusConnection::sessionBus().connect(QStringLiteral("org.freedesktop.Notifications"),
                                           QStringLiteral("/org/freedesktop/Notifications"),
                                           QStringLiteral("org.freedesktop.Notifications"),
@@ -103,14 +107,16 @@ NotificationsManager::postNotification(const mtx::responses::Notification &notif
     QString template_ = getMessageTemplate(notification);
     // TODO: decrypt this message if the decryption setting is on in the UserSettings
     if (std::holds_alternative<mtx::events::EncryptedEvent<mtx::events::msg::Encrypted>>(
-          notification.event)) {
+          notification.event) ||
+        !template_.contains("%2")) {
         postNotif(template_);
         return;
     }
 
     if (hasMarkup_) {
         if (hasImages_ &&
-            mtx::accessors::msg_type(notification.event) == mtx::events::MessageType::Image) {
+            (mtx::accessors::msg_type(notification.event) == mtx::events::MessageType::Image ||
+             mtx::accessors::event_type(notification.event) == mtx::events::EventType::Sticker)) {
             MxcImageProvider::download(
               QString::fromStdString(mtx::accessors::url(notification.event))
                 .remove(QStringLiteral("mxc://")),
@@ -190,11 +196,14 @@ NotificationsManager::systemPostNotification(const QString &room_id,
     // The list of actions has always the action name and then a localized version of that
     // action. Currently we just use an empty string for that.
     // TODO(Nico): Look into what to actually put there.
-    argumentList << (QStringList(QStringLiteral("default"))
-                     << QLatin1String("") << QStringLiteral("inline-reply")
-                     << QLatin1String("")); // actions
-    argumentList << hints;                  // hints
-    argumentList << (int)-1;                // timeout in ms
+    QStringList actions;
+    actions << QStringList(QStringLiteral("default")) << QLatin1String("");
+    if (!room_id.isEmpty()) {
+        actions << QStringLiteral("inline-reply") << QLatin1String("");
+    }
+    argumentList << actions; // actions
+    argumentList << hints;   // hints
+    argumentList << (int)-1; // timeout in ms
 
     QDBusPendingCall call = dbus.asyncCallWithArgumentList(QStringLiteral("Notify"), argumentList);
     auto watcher          = new QDBusPendingCallWatcher{call, this};
@@ -256,6 +265,14 @@ NotificationsManager::actionInvoked(uint id, QString action)
     }
 }
 
+// receive a wayland activation token from the notification manager
+void
+NotificationsManager::activationToken(uint, QString action)
+{
+    nhlog::net()->debug("Got activation token for notification");
+    qputenv("XDG_ACTIVATION_TOKEN", action.toUtf8());
+}
+
 void
 NotificationsManager::notificationReplied(uint id, QString reply)
 {
