@@ -1,11 +1,10 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "Manager.h"
 
+#include <QCoreApplication>
 #include <QRegularExpression>
 #include <QTextDocumentFragment>
 
@@ -21,7 +20,42 @@
 static QString
 formatNotification(const mtx::responses::Notification &notification)
 {
-    return utils::stripReplyFallbacks(notification.event, {}, {}).quoted_body;
+    auto fallbacks = utils::stripReplyFallbacks(notification.event, {}, {});
+
+    bool containsSpoiler = fallbacks.quoted_formatted_body.contains("<span data-mx-spoiler");
+    if (containsSpoiler)
+        return QCoreApplication::translate("macosNotification", "Message contains spoiler.");
+    else
+        return fallbacks.quoted_body;
+}
+
+NotificationsManager::NotificationsManager(QObject *parent)
+  : QObject(parent)
+{
+    // Putting these here to pass along since I'm not sure how
+    // our translate step interacts with .mm files
+    respondStr  = QObject::tr("Respond");
+    sendStr     = QObject::tr("Send");
+    placeholder = QObject::tr("Write a message...");
+
+    connect(
+      this,
+      &NotificationsManager::systemPostNotificationCb,
+      this,
+      [this](const QString &room_id,
+             const QString &event_id,
+             const QString &roomName,
+             const QString &text,
+             const QImage &) {
+          objCxxPostNotification(roomName,
+                                 room_id,
+                                 event_id,
+                                 text,
+                                 /*const QString &informativeText*/ "",
+                                 "",
+                                 true);
+      },
+      Qt::QueuedConnection);
 }
 
 void
@@ -42,12 +76,6 @@ NotificationsManager::postNotification(const mtx::responses::Notification &notif
                                &notification.event) != nullptr;
     const auto isReply = utils::isReply(notification.event);
 
-    // Putting these here to pass along since I'm not sure how
-    // our translate step interacts with .mm files
-    const auto respondStr  = QObject::tr("Respond");
-    const auto sendStr     = QObject::tr("Send");
-    const auto placeholder = QObject::tr("Write a message...");
-
     auto playSound = false;
 
     if (std::find(notification.actions.begin(),
@@ -61,16 +89,7 @@ NotificationsManager::postNotification(const mtx::responses::Notification &notif
         const QString messageInfo = (isReply ? tr("%1 replied with an encrypted message")
                                              : tr("%1 sent an encrypted message"))
                                       .arg(sender);
-        objCxxPostNotification(room_name,
-                               room_id,
-                               event_id,
-                               messageInfo,
-                               "",
-                               "",
-                               respondStr,
-                               sendStr,
-                               placeholder,
-                               playSound);
+        objCxxPostNotification(room_name, room_id, event_id, messageInfo, "", "", playSound);
     } else {
         const QString messageInfo =
           (isReply ? tr("%1 replied to a message") : tr("%1 sent a message")).arg(sender);
@@ -78,25 +97,14 @@ NotificationsManager::postNotification(const mtx::responses::Notification &notif
             MxcImageProvider::download(
               QString::fromStdString(mtx::accessors::url(notification.event)).remove("mxc://"),
               QSize(200, 80),
-              [this,
-               notification,
-               room_name,
-               room_id,
-               event_id,
-               messageInfo,
-               respondStr,
-               sendStr,
-               placeholder,
-               playSound](QString, QSize, QImage, QString imgPath) {
+              [this, notification, room_name, room_id, event_id, messageInfo, playSound](
+                QString, QSize, QImage, QString imgPath) {
                   objCxxPostNotification(room_name,
                                          room_id,
                                          event_id,
                                          messageInfo,
                                          formatNotification(notification),
                                          imgPath,
-                                         respondStr,
-                                         sendStr,
-                                         placeholder,
                                          playSound);
               });
         else
@@ -106,9 +114,6 @@ NotificationsManager::postNotification(const mtx::responses::Notification &notif
                                    messageInfo,
                                    formatNotification(notification),
                                    "",
-                                   respondStr,
-                                   sendStr,
-                                   placeholder,
                                    playSound);
     }
 }
