@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -16,25 +14,29 @@
 #include <QMediaPlayer>
 #include <QMimeData>
 #include <QMimeDatabase>
+#include <QRegularExpression>
 #include <QStandardPaths>
 #include <QTextBoundaryFinder>
+#include <QVideoFrame>
+#include <QVideoSink>
+
+#include <nlohmann/json.hpp>
 
-#include <QRegularExpression>
 #include <mtx/responses/common.hpp>
 #include <mtx/responses/media.hpp>
 
 #include "Cache.h"
 #include "Cache_p.h"
 #include "ChatPage.h"
-#include "CombinedImagePackModel.h"
-#include "Config.h"
 #include "EventAccessors.h"
 #include "Logging.h"
+#include "MainWindow.h"
 #include "MatrixClient.h"
 #include "TimelineModel.h"
 #include "TimelineViewManager.h"
 #include "UserSettingsPage.h"
 #include "Utils.h"
+#include "ui/UserProfile.h"
 
 #include "blurhash.hpp"
 
@@ -55,7 +57,7 @@ threadFallbackEventId(const std::string &room_id, const std::string &thread_id)
     for (const auto &[index, event_id] : orderedEvents) {
         (void)index;
         if (auto event = cache::client()->getEvent(room_id, event_id)) {
-            if (mtx::accessors::relations(event->data).thread() == thread_id)
+            if (mtx::accessors::relations(event.value()).thread() == thread_id)
                 return std::string(event_id);
         }
     }
@@ -73,56 +75,7 @@ MediaUpload::thumbnailDataUrl() const
     buffer.open(QIODevice::WriteOnly);
     thumbnail_.save(&buffer, "PNG");
     QString base64 = QString::fromUtf8(byteArray.toBase64());
-    return QString("data:image/png;base64,") + base64;
-}
-
-bool
-InputVideoSurface::present(const QVideoFrame &frame)
-{
-    QImage::Format format = QVideoFrame::imageFormatFromPixelFormat(frame.pixelFormat());
-
-    if (format == QImage::Format_Invalid) {
-        emit newImage({});
-        return false;
-    } else {
-        QVideoFrame frametodraw(frame);
-
-        if (!frametodraw.map(QAbstractVideoBuffer::ReadOnly)) {
-            emit newImage({});
-            return false;
-        }
-
-        // this is a shallow operation. it just refer the frame buffer
-        QImage image(qAsConst(frametodraw).bits(),
-                     frametodraw.width(),
-                     frametodraw.height(),
-                     frametodraw.bytesPerLine(),
-                     format);
-        image.detach();
-
-        frametodraw.unmap();
-
-        emit newImage(std::move(image));
-        return true;
-    }
-}
-
-QList<QVideoFrame::PixelFormat>
-InputVideoSurface::supportedPixelFormats(QAbstractVideoBuffer::HandleType type) const
-{
-    if (type == QAbstractVideoBuffer::NoHandle) {
-        return {
-          QVideoFrame::Format_ARGB32,
-          QVideoFrame::Format_ARGB32_Premultiplied,
-          QVideoFrame::Format_RGB24,
-          QVideoFrame::Format_BGR24,
-          QVideoFrame::Format_RGB32,
-          QVideoFrame::Format_RGB565,
-          QVideoFrame::Format_RGB555,
-        };
-    } else {
-        return {};
-    }
+    return QStringLiteral("data:image/png;base64,") + base64;
 }
 
 bool
@@ -150,6 +103,7 @@ InputBar::insertMimeData(const QMimeData *md)
 
     nhlog::ui()->debug("Got mime formats: {}",
                        md->formats().join(QStringLiteral(", ")).toStdString());
+    nhlog::ui()->debug("Has image: {}", md->hasImage());
     const auto formats = md->formats().filter(QStringLiteral("/"));
     const auto image   = formats.filter(QStringLiteral("image/"), Qt::CaseInsensitive);
     const auto audio   = formats.filter(QStringLiteral("audio/"), Qt::CaseInsensitive);
@@ -226,30 +180,135 @@ InputBar::insertMimeData(const QMimeData *md)
 }
 
 void
-InputBar::updateAtRoom(const QString &t)
+InputBar::addMention(QString mention, QString text)
 {
-    bool roomMention = false;
+    if (!mentions_.contains(mention)) {
+        mentions_.push_back(mention);
+        mentionTexts_.push_back(text);
 
-    if (t.size() > 4) {
-        QTextBoundaryFinder finder(QTextBoundaryFinder::BoundaryType::Word, t);
+        emit mentionsChanged();
+        if (mention == u"@room") {
+            this->containsAtRoom_ = true;
+        }
+    }
+}
 
-        finder.toStart();
-        do {
-            auto start = finder.position();
-            finder.toNextBoundary();
-            auto end = finder.position();
-            if (start > 0 && end - start >= 4 &&
-                t.mid(start, end - start) == QLatin1String("room") &&
-                t.at(start - 1) == QChar('@')) {
-                roomMention = true;
-                break;
+void
+InputBar::removeMention(QString mention)
+{
+    if (auto idx = mentions_.indexOf(mention); idx != -1) {
+        mentions_.removeAt(idx);
+        mentionTexts_.removeAt(idx);
+        emit mentionsChanged();
+        if (mention == u"@room") {
+            this->containsAtRoom_ = false;
+        }
+    }
+}
+
+void
+InputBar::updateTextContentProperties(const QString &t, bool charDeleted)
+{
+    auto containsRoomMention = [](QStringView text) {
+        // check for @room
+        bool roomMention = false;
+        if (text.size() > 4) {
+            QTextBoundaryFinder finder(QTextBoundaryFinder::BoundaryType::Word, text);
+
+            finder.toStart();
+            do {
+                auto start = finder.position();
+                finder.toNextBoundary();
+                auto end = finder.position();
+                if (start > 0 && end - start >= 4 &&
+                    text.mid(start, end - start) == QStringView(u"room") &&
+                    text.at(start - 1) == QChar('@')) {
+                    roomMention = true;
+                    break;
+                }
+            } while (finder.position() < text.size());
+        }
+        return roomMention;
+    };
+
+    if (charDeleted) {
+        for (qsizetype idx = 0; idx < mentions_.size();) {
+            if (!t.contains(mentionTexts_.at(idx))) {
+                removeMention(mentions_.at(idx));
+            } else {
+                idx++;
             }
-        } while (finder.position() < t.size());
+        }
     }
 
+    auto roomMention = containsRoomMention(t) && this->room->permissions()->canPingRoom();
+
     if (roomMention != this->containsAtRoom_) {
-        this->containsAtRoom_ = roomMention;
-        emit containsAtRoomChanged();
+        if (roomMention)
+            addMention(QStringLiteral(u"@room"), QStringLiteral(u"@room"));
+        else
+            removeMention(QStringLiteral(u"@room"));
+    }
+
+    // check for invalid commands
+    auto commandName = getCommandAndArgs(t).first;
+    static const QSet<QString> validCommands{QStringLiteral("me"),
+                                             QStringLiteral("react"),
+                                             QStringLiteral("join"),
+                                             QStringLiteral("knock"),
+                                             QStringLiteral("part"),
+                                             QStringLiteral("leave"),
+                                             QStringLiteral("invite"),
+                                             QStringLiteral("kick"),
+                                             QStringLiteral("ban"),
+                                             QStringLiteral("unban"),
+                                             QStringLiteral("redact"),
+                                             QStringLiteral("roomnick"),
+                                             QStringLiteral("shrug"),
+                                             QStringLiteral("fliptable"),
+                                             QStringLiteral("unfliptable"),
+                                             QStringLiteral("sovietflip"),
+                                             QStringLiteral("clear-timeline"),
+                                             QStringLiteral("reset-state"),
+                                             QStringLiteral("rotate-megolm-session"),
+                                             QStringLiteral("md"),
+                                             QStringLiteral("cmark"),
+                                             QStringLiteral("plain"),
+                                             QStringLiteral("rainbow"),
+                                             QStringLiteral("rainbowme"),
+                                             QStringLiteral("notice"),
+                                             QStringLiteral("rainbownotice"),
+                                             QStringLiteral("confetti"),
+                                             QStringLiteral("rainbowconfetti"),
+                                             QStringLiteral("rainfall"),
+                                             QStringLiteral("msgtype"),
+                                             QStringLiteral("glitch"),
+                                             QStringLiteral("gradualglitch"),
+                                             QStringLiteral("goto"),
+                                             QStringLiteral("converttodm"),
+                                             QStringLiteral("converttoroom"),
+                                             QStringLiteral("ignore"),
+                                             QStringLiteral("unignore")};
+    bool hasInvalidCommand    = !commandName.isNull() && !validCommands.contains(commandName);
+    bool hasIncompleteCommand = hasInvalidCommand && '/' + commandName == t;
+
+    bool signalsChanged{false};
+    if (containsInvalidCommand_ != hasInvalidCommand) {
+        containsInvalidCommand_ = hasInvalidCommand;
+        signalsChanged          = true;
+    }
+    if (containsIncompleteCommand_ != hasIncompleteCommand) {
+        containsIncompleteCommand_ = hasIncompleteCommand;
+        signalsChanged             = true;
+    }
+    if (currentCommand_ != commandName) {
+        currentCommand_ = commandName;
+        signalsChanged  = true;
+    }
+    if (signalsChanged) {
+        emit currentCommandChanged();
+        emit containsInvalidCommandChanged();
+        emit containsIncompleteCommandChanged();
     }
 }
 
@@ -265,7 +324,7 @@ InputBar::setText(const QString &newText)
     if (history_.size() == INPUT_HISTORY_SIZE)
         history_.pop_back();
 
-    updateAtRoom(QLatin1String(""));
+    updateTextContentProperties(newText, true);
     emit textChanged(newText);
 }
 void
@@ -279,14 +338,15 @@ InputBar::updateState(int selectionStart_,
     else
         startTyping();
 
-    if (text_ != text()) {
+    auto oldText = text();
+    if (text_ != oldText) {
         if (history_.empty())
             history_.push_front(text_);
         else
             history_.front() = text_;
         history_index_ = 0;
 
-        updateAtRoom(text_);
+        updateTextContentProperties(text_, text_.size() < oldText.size());
         // disabled, as it moves the cursor to the end
         // emit textChanged(text_);
     }
@@ -314,7 +374,7 @@ InputBar::previousText()
     else if (text().isEmpty())
         history_index_--;
 
-    updateAtRoom(text());
+    updateTextContentProperties(text());
     return text();
 }
 
@@ -325,7 +385,7 @@ InputBar::nextText()
     if (history_index_ >= INPUT_HISTORY_SIZE)
         history_index_ = 0;
 
-    updateAtRoom(text());
+    updateTextContentProperties(text());
     return text();
 }
 
@@ -343,20 +403,12 @@ InputBar::send()
 
     auto wasEdit = !room->edit().isEmpty();
 
-    if (text().startsWith('/')) {
-        int command_end = text().indexOf(QRegularExpression(QStringLiteral("\\s")));
-        if (command_end == -1)
-            command_end = text().size();
-        auto name = text().mid(1, command_end - 1);
-        auto args = text().mid(command_end + 1);
-        if (name.isEmpty() || name == QLatin1String("/")) {
-            message(args);
-        } else {
-            command(name, args);
-        }
-    } else {
+    auto [commandName, args] = getCommandAndArgs();
+    updateTextContentProperties(text());
+    if (containsIncompleteCommand_)
+        return;
+    if (commandName.isEmpty() || !command(commandName, args))
         message(text());
-    }
 
     if (!wasEdit) {
         history_.push_front(QLatin1String(""));
@@ -368,13 +420,15 @@ void
 InputBar::openFileSelection()
 {
     const QString homeFolder = QStandardPaths::writableLocation(QStandardPaths::HomeLocation);
-    const auto fileName =
-      QFileDialog::getOpenFileName(nullptr, tr("Select a file"), homeFolder, tr("All Files (*)"));
+    const QStringList fileNames =
+      QFileDialog::getOpenFileNames(nullptr, tr("Select file(s)"), homeFolder, tr("All Files (*)"));
 
-    if (fileName.isEmpty())
+    if (fileNames.isEmpty())
         return;
 
-    startUploadFromPath(fileName);
+    ChatPage::instance()->timelineManager()->focusMessageInput();
+    for (const auto &fileName : fileNames)
+        startUploadFromPath(fileName);
 }
 
 QString
@@ -408,8 +462,7 @@ replaceMatrixToMarkdownLink(QString input)
         int newline   = input.indexOf('\n', endOfName);
         if (endOfLink > endOfName && (newline == -1 || endOfLink < newline)) {
             auto name = input.mid(startOfName + 1, endOfName - startOfName - 1);
-            name.replace("\\[", "[");
-            name.replace("\\]", "]");
+            name.remove(QChar(u'\\'), Qt::CaseSensitive);
             input.replace(startOfName, endOfLink - startOfName + 1, name);
             replaced = true;
         }
@@ -443,6 +496,35 @@ InputBar::generateRelations() const
     return relations;
 }
 
+mtx::common::Mentions
+InputBar::generateMentions()
+{
+    std::vector<std::string> userMentions;
+    for (const auto &m : mentions_)
+        if (m != u"@room")
+            userMentions.push_back(m.toStdString());
+
+    if (!room->reply().isEmpty()) {
+        auto replyToSender =
+          room->dataById(room->reply(), TimelineModel::Roles::UserId, "").toString().toStdString();
+        if (!replyToSender.empty() &&
+            std::ranges::find(userMentions, replyToSender) == userMentions.end()) {
+            userMentions.push_back(replyToSender);
+        }
+    }
+
+    auto mention = mtx::common::Mentions{
+      .user_ids = userMentions,
+      .room     = containsAtRoom_,
+    };
+
+    // this->containsAtRoom_ = false;
+    // this->mentions_.clear();
+    // this->mentionTexts_.clear();
+
+    return mention;
+}
+
 void
 InputBar::message(const QString &msg, MarkdownOverride useMarkdown, bool rainbowify)
 {
@@ -457,11 +539,11 @@ InputBar::message(const QString &msg, MarkdownOverride useMarkdown, bool rainbow
         text.body = replaceMatrixToMarkdownLink(msg.trimmed()).toStdString();
 
         // Don't send formatted_body, when we don't need to
-        // Specifically, if it includes no html tag and no newlines (which behave differently in
-        // formatted bodies). Probably we forgot something, so this might need to expand at some
-        // point.
+        // Specifically, if it includes no html tag and no newlines or
+        // backslashes (which behave differently in formatted bodies). Probably
+        // we forgot something, so this might need to expand at some point.
         if (text.formatted_body.find('<') == std::string::npos &&
-            text.body.find('\n') == std::string::npos)
+            text.body.find('\n') == std::string::npos && text.body.find('\\') == std::string::npos)
             text.formatted_body = "";
         else
             text.format = "org.matrix.custom.html";
@@ -475,6 +557,7 @@ InputBar::message(const QString &msg, MarkdownOverride useMarkdown, bool rainbow
         text.format = "org.matrix.custom.html";
     }
 
+    text.mentions  = generateMentions();
     text.relations = generateRelations();
     if (!room->reply().isEmpty() && room->thread().isEmpty() && room->edit().isEmpty()) {
         auto related = room->relatedInfo(room->reply());
@@ -486,8 +569,8 @@ InputBar::message(const QString &msg, MarkdownOverride useMarkdown, bool rainbow
         if (!related.quoted_user.startsWith("@room:")) {
             QString body;
             bool firstLine = true;
-            auto lines     = related.quoted_body.splitRef(u'\n');
-            for (auto line : qAsConst(lines)) {
+            auto lines     = QStringView(related.quoted_body).split(u'\n');
+            for (auto line : std::as_const(lines)) {
                 if (firstLine) {
                     firstLine = false;
                     body      = QStringLiteral("> <%1> %2\n").arg(related.quoted_user, line);
@@ -531,6 +614,7 @@ InputBar::emote(const QString &msg, bool rainbowify)
         emote.body = replaceMatrixToMarkdownLink(msg.trimmed()).toStdString();
     }
 
+    emote.mentions  = generateMentions();
     emote.relations = generateRelations();
 
     room->sendMessageEvent(emote, mtx::events::EventType::RoomMessage);
@@ -551,6 +635,7 @@ InputBar::notice(const QString &msg, bool rainbowify)
         notice.body = replaceMatrixToMarkdownLink(msg.trimmed()).toStdString();
     }
 
+    notice.mentions  = generateMentions();
     notice.relations = generateRelations();
 
     room->sendMessageEvent(notice, mtx::events::EventType::RoomMessage);
@@ -561,8 +646,9 @@ InputBar::confetti(const QString &body, bool rainbowify)
 {
     auto html = utils::markdownToHtml(body, rainbowify);
 
-    mtx::events::msg::Confetti confetti;
-    confetti.body = body.trimmed().toStdString();
+    mtx::events::msg::ElementEffect confetti;
+    confetti.msgtype = "nic.custom.confetti";
+    confetti.body    = body.trimmed().toStdString();
 
     if (html != body.trimmed().toHtmlEscaped() &&
         ChatPage::instance()->userSettings()->markdown()) {
@@ -572,11 +658,62 @@ InputBar::confetti(const QString &body, bool rainbowify)
         confetti.body = replaceMatrixToMarkdownLink(body.trimmed()).toStdString();
     }
 
+    confetti.mentions  = generateMentions();
     confetti.relations = generateRelations();
 
     room->sendMessageEvent(confetti, mtx::events::EventType::RoomMessage);
 }
 
+void
+InputBar::rainfall(const QString &body)
+{
+    auto html = utils::markdownToHtml(body);
+
+    mtx::events::msg::Unknown rain;
+    rain.msgtype = "io.element.effect.rainfall";
+    rain.body    = body.trimmed().toStdString();
+
+    if (html != body.trimmed().toHtmlEscaped() &&
+        ChatPage::instance()->userSettings()->markdown()) {
+        nlohmann::json j;
+        j["formatted_body"] = html.toStdString();
+        j["format"]         = "org.matrix.custom.html";
+        rain.content        = j.dump();
+        // Remove markdown links by completer
+        rain.body = replaceMatrixToMarkdownLink(body.trimmed()).toStdString();
+    }
+
+    rain.mentions  = generateMentions();
+    rain.relations = generateRelations();
+
+    room->sendMessageEvent(rain, mtx::events::EventType::RoomMessage);
+}
+
+void
+InputBar::customMsgtype(const QString &msgtype, const QString &body)
+{
+    auto html = utils::markdownToHtml(body);
+
+    mtx::events::msg::Unknown msg;
+    msg.msgtype = msgtype.toStdString();
+    msg.body    = body.trimmed().toStdString();
+
+    if (html != body.trimmed().toHtmlEscaped() &&
+        ChatPage::instance()->userSettings()->markdown()) {
+        nlohmann::json j;
+        j["formatted_body"] = html.toStdString();
+        j["format"]         = "org.matrix.custom.html";
+        msg.content         = j.dump();
+        // Remove markdown links by completer
+        msg.body = replaceMatrixToMarkdownLink(body.trimmed()).toStdString();
+    }
+
+    msg.mentions  = generateMentions();
+    msg.relations = generateRelations();
+
+    room->sendMessageEvent(msg, mtx::events::EventType::RoomMessage);
+}
+
 void
 InputBar::image(const QString &filename,
                 const std::optional<mtx::crypto::EncryptedFile> &file,
@@ -615,6 +752,7 @@ InputBar::image(const QString &filename,
         image.info.thumbnail_info.mimetype = "image/png";
     }
 
+    image.mentions  = generateMentions();
     image.relations = generateRelations();
 
     room->sendMessageEvent(image, mtx::events::EventType::RoomMessage);
@@ -637,6 +775,7 @@ InputBar::file(const QString &filename,
     else
         file.url = url.toStdString();
 
+    file.mentions  = generateMentions();
     file.relations = generateRelations();
 
     room->sendMessageEvent(file, mtx::events::EventType::RoomMessage);
@@ -664,6 +803,7 @@ InputBar::audio(const QString &filename,
     else
         audio.url = url.toStdString();
 
+    audio.mentions  = generateMentions();
     audio.relations = generateRelations();
 
     room->sendMessageEvent(audio, mtx::events::EventType::RoomMessage);
@@ -713,37 +853,71 @@ InputBar::video(const QString &filename,
         video.info.thumbnail_info.mimetype = "image/png";
     }
 
+    video.mentions  = generateMentions();
     video.relations = generateRelations();
 
     room->sendMessageEvent(video, mtx::events::EventType::RoomMessage);
 }
 
-void
-InputBar::sticker(CombinedImagePackModel *model, int row)
+QPair<QString, QString>
+InputBar::getCommandAndArgs(const QString &currentText) const
 {
-    if (!model || row < 0)
-        return;
+    if (!currentText.startsWith('/'))
+        return {{}, currentText};
 
-    auto img = model->imageAt(row);
+    static QRegularExpression spaceRegex(QStringLiteral("\\s"));
 
-    mtx::events::msg::StickerImage sticker{};
-    sticker.info = img.info.value_or(mtx::common::ImageInfo{});
-    sticker.url  = img.url;
-    sticker.body = img.body.empty() ? model->shortcodeAt(row).toStdString() : img.body;
-
-    // workaround for https://github.com/vector-im/element-ios/issues/2353
-    sticker.info.thumbnail_url           = sticker.url;
-    sticker.info.thumbnail_info.mimetype = sticker.info.mimetype;
-    sticker.info.thumbnail_info.size     = sticker.info.size;
-    sticker.info.thumbnail_info.h        = sticker.info.h;
-    sticker.info.thumbnail_info.w        = sticker.info.w;
-
-    sticker.relations = generateRelations();
-
-    room->sendMessageEvent(sticker, mtx::events::EventType::Sticker);
+    int command_end = currentText.indexOf(spaceRegex);
+    if (command_end == -1)
+        command_end = currentText.size();
+    auto name = currentText.mid(1, command_end - 1);
+    auto args = currentText.mid(command_end + 1);
+    if (name.isEmpty() || name == QLatin1String("/")) {
+        return {{}, currentText};
+    } else {
+        return {name, args};
+    }
 }
 
 void
+InputBar::sticker(QStringList descriptor)
+{
+    if (descriptor.size() != 3)
+        return;
+
+    auto originalPacks = cache::client()->getImagePacks(room->roomId().toStdString(), true);
+
+    auto source_room = descriptor[0].toStdString();
+    auto state_key   = descriptor[1].toStdString();
+    auto short_code  = descriptor[2].toStdString();
+
+    for (auto &pack : originalPacks) {
+        if (pack.source_room == source_room && pack.state_key == state_key &&
+            pack.pack.images.contains(short_code)) {
+            auto img = pack.pack.images.at(short_code);
+
+            mtx::events::msg::StickerImage sticker{};
+            sticker.info = img.info.value_or(mtx::common::ImageInfo{});
+            sticker.url  = img.url;
+            sticker.body = img.body.empty() ? short_code : img.body;
+
+            // workaround for https://github.com/vector-im/element-ios/issues/2353
+            sticker.info.thumbnail_url           = sticker.url;
+            sticker.info.thumbnail_info.mimetype = sticker.info.mimetype;
+            sticker.info.thumbnail_info.size     = sticker.info.size;
+            sticker.info.thumbnail_info.h        = sticker.info.h;
+            sticker.info.thumbnail_info.w        = sticker.info.w;
+
+            sticker.mentions  = generateMentions();
+            sticker.relations = generateRelations();
+
+            room->sendMessageEvent(sticker, mtx::events::EventType::Sticker);
+            break;
+        }
+    }
+}
+
+bool
 InputBar::command(const QString &command, QString args)
 {
     if (command == QLatin1String("me")) {
@@ -795,7 +969,7 @@ InputBar::command(const QString &command, QString args)
                                       err->matrix_error.error);
           });
     } else if (command == QLatin1String("shrug")) {
-        message("¯\\_(ツ)_/¯" + (args.isEmpty() ? QLatin1String("") : " " + args));
+        message("¯\\\\\\_(ツ)\\_/¯" + (args.isEmpty() ? QLatin1String("") : " " + args));
     } else if (command == QLatin1String("fliptable")) {
         message(QStringLiteral("(╯°□°)╯︵ ┻━┻"));
     } else if (command == QLatin1String("unfliptable")) {
@@ -826,21 +1000,29 @@ InputBar::command(const QString &command, QString args)
         confetti(args, false);
     } else if (command == QLatin1String("rainbowconfetti")) {
         confetti(args, true);
+    } else if (command == QLatin1String("rainfall")) {
+        rainfall(args);
+    } else if (command == QLatin1String("msgtype")) {
+        customMsgtype(args.section(' ', 0, 0), args.section(' ', 1, -1));
+    } else if (command == QLatin1String("glitch")) {
+        message(utils::glitchText(args));
+    } else if (command == QLatin1String("gradualglitch")) {
+        message(utils::graduallyGlitchText(args));
     } else if (command == QLatin1String("goto")) {
         // Goto has three different modes:
         // 1 - Going directly to a given event ID
         if (args[0] == '$') {
             room->showEvent(args);
-            return;
+            return true;
         }
         // 2 - Going directly to a given message index
         if (args[0] >= '0' && args[0] <= '9') {
             room->showEvent(args);
-            return;
+            return true;
         }
         // 3 - Matrix URI handler, as if you clicked the URI
         if (ChatPage::instance()->handleMatrixUri(args)) {
-            return;
+            return true;
         }
         nhlog::net()->error("Could not resolve goto: {}", args.toStdString());
     } else if (command == QLatin1String("converttodm")) {
@@ -848,7 +1030,32 @@ InputBar::command(const QString &command, QString args)
                                 cache::getMembers(this->room->roomId().toStdString(), 0, -1));
     } else if (command == QLatin1String("converttoroom")) {
         utils::removeDirectFromRoom(this->room->roomId());
+    } else if (command == QLatin1String("ignore")) {
+        this->toggleIgnore(args, true);
+    } else if (command == QLatin1String("unignore")) {
+        this->toggleIgnore(args, false);
+    } else {
+        return false;
     }
+
+    return true;
+}
+
+void
+InputBar::toggleIgnore(const QString &user, const bool ignored)
+{
+    UserProfile *profile = new UserProfile(QString(), user, TimelineViewManager::instance());
+    connect(profile, &UserProfile::failedToFetchProfile, [user, profile] {
+        MainWindow::instance()->showNotification(tr("Failed to fetch user %1").arg(user));
+        profile->deleteLater();
+    });
+
+    connect(
+      profile, &UserProfile::globalUsernameRetrieved, [profile, ignored](const QString &user_id) {
+          Q_UNUSED(user_id)
+          profile->setIgnored(ignored);
+          profile->deleteLater();
+      });
 }
 
 MediaUpload::MediaUpload(std::unique_ptr<QIODevice> source_,
@@ -899,83 +1106,83 @@ MediaUpload::MediaUpload(std::unique_ptr<QIODevice> source_,
         blurhash_ =
           QString::fromStdString(blurhash::encode(data_.data(), img.width(), img.height(), 4, 3));
     } else if (mimeClass_ == u"video" || mimeClass_ == u"audio") {
-        auto mediaPlayer = new QMediaPlayer(
-          this,
-          mimeClass_ == u"video" ? QFlags{QMediaPlayer::VideoSurface} : QMediaPlayer::Flags{});
-        mediaPlayer->setMuted(true);
+        auto mediaPlayer = new QMediaPlayer(this);
+        mediaPlayer->setAudioOutput(nullptr);
 
         if (mimeClass_ == u"video") {
-            auto newSurface = new InputVideoSurface(this);
-            connect(
-              newSurface, &InputVideoSurface::newImage, this, [this, mediaPlayer](QImage img) {
-                  if (img.size().isEmpty())
-                      return;
+            auto newSurface = new QVideoSink(this);
+            connect(newSurface,
+                    &QVideoSink::videoFrameChanged,
+                    this,
+                    [this, mediaPlayer](const QVideoFrame &frame) {
+                        QImage img = frame.toImage();
+                        if (img.size().isEmpty())
+                            return;
 
-                  mediaPlayer->stop();
+                        mediaPlayer->stop();
 
-                  auto orientation = mediaPlayer->metaData(QMediaMetaData::Orientation).toInt();
-                  if (orientation == 90 || orientation == 270 || orientation == 180) {
-                      img =
-                        img.transformed(QTransform().rotate(orientation), Qt::SmoothTransformation);
-                  }
+                        auto orientation =
+                          mediaPlayer->metaData().value(QMediaMetaData::Orientation).toInt();
+                        if (orientation == 90 || orientation == 270 || orientation == 180) {
+                            img = img.transformed(QTransform().rotate(orientation),
+                                                  Qt::SmoothTransformation);
+                        }
 
-                  nhlog::ui()->debug("Got image {}x{}", img.width(), img.height());
+                        nhlog::ui()->debug("Got image {}x{}", img.width(), img.height());
 
-                  this->setThumbnail(img);
+                        this->setThumbnail(img);
 
-                  if (!dimensions_.isValid())
-                      this->dimensions_ = img.size();
+                        if (!dimensions_.isValid())
+                            this->dimensions_ = img.size();
 
-                  if (img.height() > 200 && img.width() > 360)
-                      img = img.scaled(360, 200, Qt::KeepAspectRatioByExpanding);
-                  std::vector<unsigned char> data_;
-                  for (int y = 0; y < img.height(); y++) {
-                      for (int x = 0; x < img.width(); x++) {
-                          auto p = img.pixel(x, y);
-                          data_.push_back(static_cast<unsigned char>(qRed(p)));
-                          data_.push_back(static_cast<unsigned char>(qGreen(p)));
-                          data_.push_back(static_cast<unsigned char>(qBlue(p)));
-                      }
-                  }
-                  blurhash_ = QString::fromStdString(
-                    blurhash::encode(data_.data(), img.width(), img.height(), 4, 3));
-              });
+                        if (img.height() > 200 && img.width() > 360)
+                            img = img.scaled(360, 200, Qt::KeepAspectRatioByExpanding);
+                        std::vector<unsigned char> data_;
+                        for (int y = 0; y < img.height(); y++) {
+                            for (int x = 0; x < img.width(); x++) {
+                                auto p = img.pixel(x, y);
+                                data_.push_back(static_cast<unsigned char>(qRed(p)));
+                                data_.push_back(static_cast<unsigned char>(qGreen(p)));
+                                data_.push_back(static_cast<unsigned char>(qBlue(p)));
+                            }
+                        }
+                        blurhash_ = QString::fromStdString(
+                          blurhash::encode(data_.data(), img.width(), img.height(), 4, 3));
+                    });
             mediaPlayer->setVideoOutput(newSurface);
         }
 
         connect(mediaPlayer,
-                qOverload<QMediaPlayer::Error>(&QMediaPlayer::error),
+                &QMediaPlayer::errorOccurred,
                 this,
-                [mediaPlayer](QMediaPlayer::Error error) {
+                [](QMediaPlayer::Error error, QString errorString) {
                     nhlog::ui()->debug("Media player error {} and errorStr {}",
-                                       error,
-                                       mediaPlayer->errorString().toStdString());
+                                       static_cast<int>(error),
+                                       errorString.toStdString());
                 });
         connect(mediaPlayer,
                 &QMediaPlayer::mediaStatusChanged,
                 [mediaPlayer](QMediaPlayer::MediaStatus status) {
-                    nhlog::ui()->debug(
-                      "Media player status {} and error {}", status, mediaPlayer->error());
+                    nhlog::ui()->debug("Media player status {} and error {}",
+                                       static_cast<int>(status),
+                                       static_cast<int>(mediaPlayer->error()));
                 });
-        connect(mediaPlayer,
-                qOverload<const QString &, const QVariant &>(&QMediaPlayer::metaDataChanged),
-                this,
-                [this, mediaPlayer](const QString &t, const QVariant &) {
-                    nhlog::ui()->debug("Got metadata {}", t.toStdString());
+        connect(mediaPlayer, &QMediaPlayer::metaDataChanged, this, [this, mediaPlayer]() {
+            nhlog::ui()->debug("Got metadata {}");
 
-                    if (mediaPlayer->duration() > 0)
-                        this->duration_ = mediaPlayer->duration();
+            if (mediaPlayer->duration() > 0)
+                this->duration_ = mediaPlayer->duration();
 
-                    auto dimensions = mediaPlayer->metaData(QMediaMetaData::Resolution).toSize();
-                    if (!dimensions.isEmpty()) {
-                        dimensions_ = dimensions;
-                        auto orientation =
-                          mediaPlayer->metaData(QMediaMetaData::Orientation).toInt();
-                        if (orientation == 90 || orientation == 270) {
-                            dimensions_.transpose();
-                        }
-                    }
-                });
+            auto dimensions = mediaPlayer->metaData().value(QMediaMetaData::Resolution).toSize();
+            if (!dimensions.isEmpty()) {
+                dimensions_ = dimensions;
+                auto orientation =
+                  mediaPlayer->metaData().value(QMediaMetaData::Orientation).toInt();
+                if (orientation == 90 || orientation == 270) {
+                    dimensions_.transpose();
+                }
+            }
+        });
         connect(
           mediaPlayer, &QMediaPlayer::durationChanged, this, [this, mediaPlayer](qint64 duration) {
               if (duration > 0) {
@@ -988,8 +1195,8 @@ MediaUpload::MediaUpload(std::unique_ptr<QIODevice> source_,
 
         auto originalFile = qobject_cast<QFile *>(source.get());
 
-        mediaPlayer->setMedia(
-          QMediaContent(originalFile ? originalFile->fileName() : originalFilename_), source.get());
+        mediaPlayer->setSourceDevice(
+          source.get(), QUrl(originalFile ? originalFile->fileName() : originalFilename_));
 
         mediaPlayer->play();
     }
@@ -1299,3 +1506,5 @@ InputBar::reaction(const QString &reactedEvent, const QString &reactionKey)
         room->redactEvent(selfReactedEvent);
     }
 }
+
+#include "moc_InputBar.cpp"
