@@ -1,25 +1,24 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
-#include <QAbstractVideoSurface>
 #include <QIODevice>
 #include <QImage>
 #include <QObject>
+#include <QQmlEngine>
 #include <QSize>
 #include <QStringList>
 #include <QTimer>
 #include <QUrl>
 #include <QVariantList>
+
 #include <deque>
 #include <memory>
 
 #include <mtx/common.hpp>
-#include <mtx/responses/messages.hpp>
+#include <mtx/events/common.hpp>
 
 class TimelineModel;
 class CombinedImagePackModel;
@@ -43,28 +42,13 @@ enum class MarkdownOverride
     CMARK,
 };
 
-class InputVideoSurface final : public QAbstractVideoSurface
-{
-    Q_OBJECT
-
-public:
-    InputVideoSurface(QObject *parent)
-      : QAbstractVideoSurface(parent)
-    {
-    }
-
-    bool present(const QVideoFrame &frame) override;
-
-    QList<QVideoFrame::PixelFormat>
-    supportedPixelFormats(QAbstractVideoBuffer::HandleType type) const override;
-
-signals:
-    void newImage(QImage img);
-};
-
 class MediaUpload final : public QObject
 {
     Q_OBJECT
+
+    QML_ELEMENT
+    QML_UNCREATABLE("")
+
     Q_PROPERTY(int mediaType READ type NOTIFY mediaTypeChanged)
     // https://stackoverflow.com/questions/33422265/pass-qimage-to-qml/68554646#68554646
     Q_PROPERTY(QUrl thumbnail READ thumbnailDataUrl NOTIFY thumbnailChanged)
@@ -174,7 +158,12 @@ class InputBar final : public QObject
 {
     Q_OBJECT
     Q_PROPERTY(bool uploading READ uploading NOTIFY uploadingChanged)
-    Q_PROPERTY(bool containsAtRoom READ containsAtRoom NOTIFY containsAtRoomChanged)
+    Q_PROPERTY(
+      bool containsInvalidCommand READ containsInvalidCommand NOTIFY containsInvalidCommandChanged)
+    Q_PROPERTY(bool containsIncompleteCommand READ containsIncompleteCommand NOTIFY
+                 containsIncompleteCommandChanged)
+    Q_PROPERTY(QString currentCommand READ currentCommand NOTIFY currentCommandChanged)
+    Q_PROPERTY(QStringList mentions READ mentions NOTIFY mentionsChanged)
     Q_PROPERTY(QString text READ text NOTIFY textChanged)
     Q_PROPERTY(QVariantList uploads READ uploads NOTIFY uploadsChanged)
 
@@ -199,7 +188,45 @@ public slots:
     QString nextText();
     void setText(const QString &newText);
 
-    [[nodiscard]] bool containsAtRoom() const { return containsAtRoom_; }
+    [[nodiscard]] QStringList mentions() const { return mentions_; }
+    void addMention(QString m, QString text);
+    void removeMention(QString m);
+
+    void storeForEdit()
+    {
+        textBeforeEdit       = text();
+        mentionsBefore       = mentions_;
+        mentionTextsBefore   = mentionTexts_;
+        containsAtRoomBefore = containsAtRoom_;
+        emit mentionsChanged();
+    }
+    void restoreAfterEdit()
+    {
+        mentions_       = mentionsBefore;
+        mentionTexts_   = mentionTextsBefore;
+        containsAtRoom_ = containsAtRoomBefore;
+        mentionsBefore.clear();
+        mentionTextsBefore.clear();
+        setText(textBeforeEdit);
+        textBeforeEdit.clear();
+        emit mentionsChanged();
+    }
+    void replaceMentions(QStringList newMentions, QStringList newMentionTexts)
+    {
+        if (newMentions.size() != newMentionTexts.size())
+            return;
+
+        mentions_     = newMentions;
+        mentionTexts_ = newMentionTexts;
+
+        this->containsAtRoom_ = newMentions.contains(u"@room");
+
+        emit mentionsChanged();
+    }
+
+    bool containsInvalidCommand() const { return containsInvalidCommand_; }
+    bool containsIncompleteCommand() const { return containsIncompleteCommand_; }
+    QString currentCommand() const { return currentCommand_; }
 
     void send();
     bool tryPasteAttachment(bool fromMouse);
@@ -211,7 +238,7 @@ public slots:
                  MarkdownOverride useMarkdown = MarkdownOverride::NOT_SPECIFIED,
                  bool rainbowify              = false);
     void reaction(const QString &reactedEvent, const QString &reactionKey);
-    void sticker(CombinedImagePackModel *model, int row);
+    void sticker(QStringList descriptor);
 
     void acceptUploads();
     void declineUploads();
@@ -226,14 +253,19 @@ private slots:
 signals:
     void textChanged(QString newText);
     void uploadingChanged(bool value);
-    void containsAtRoomChanged();
+    void containsInvalidCommandChanged();
+    void mentionsChanged();
+    void containsIncompleteCommandChanged();
+    void currentCommandChanged();
     void uploadsChanged();
 
 private:
     void emote(const QString &body, bool rainbowify);
     void notice(const QString &body, bool rainbowify);
     void confetti(const QString &body, bool rainbowify);
-    void command(const QString &name, QString args);
+    void rainfall(const QString &body);
+    void customMsgtype(const QString &msgtype, const QString &body);
+    bool command(const QString &name, QString args);
     void image(const QString &filename,
                const std::optional<mtx::crypto::EncryptedFile> &file,
                const QString &url,
@@ -269,7 +301,10 @@ private:
                const QSize &thumbnailDimensions,
                const QString &blurhash);
 
+    QPair<QString, QString> getCommandAndArgs() const { return getCommandAndArgs(text()); }
+    QPair<QString, QString> getCommandAndArgs(const QString &currentText) const;
     mtx::common::Relations generateRelations() const;
+    mtx::common::Mentions generateMentions();
 
     void startUploadFromPath(const QString &path);
     void startUploadFromMimeData(const QMimeData &source, const QString &format);
@@ -282,7 +317,9 @@ private:
         }
     }
 
-    void updateAtRoom(const QString &t);
+    void updateTextContentProperties(const QString &t, bool textDeleted = false);
+
+    void toggleIgnore(const QString &user, const bool ignored);
 
     QTimer typingRefresh_;
     QTimer typingTimeout_;
@@ -290,8 +327,16 @@ private:
     std::deque<QString> history_;
     std::size_t history_index_ = 0;
     int selectionStart = 0, selectionEnd = 0, cursorPosition = 0;
-    bool uploading_      = false;
-    bool containsAtRoom_ = false;
+    bool uploading_                 = false;
+    bool containsAtRoom_            = false;
+    bool containsInvalidCommand_    = false;
+    bool containsIncompleteCommand_ = false;
+    QString currentCommand_;
+    QStringList mentions_, mentionTexts_;
+    // store stuff during edits
+    QStringList mentionsBefore, mentionTextsBefore;
+    QString textBeforeEdit;
+    bool containsAtRoomBefore = false;
 
     using UploadHandle = std::unique_ptr<MediaUpload, DeleteLaterDeleter>;
     std::vector<UploadHandle> unconfirmedUploads;
