@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -30,8 +28,6 @@ RoomlistModel::RoomlistModel(TimelineViewManager *parent)
   : QAbstractListModel(parent)
   , manager(parent)
 {
-    [[maybe_unused]] static auto id = qRegisterMetaType<RoomPreview>();
-
     connect(ChatPage::instance(), &ChatPage::decryptSidebarChanged, this, [this]() {
         auto decrypt = ChatPage::instance()->userSettings()->decryptSidebar();
         QHash<QString, QSharedPointer<TimelineModel>>::iterator i;
@@ -224,7 +220,7 @@ RoomlistModel::data(const QModelIndex &index, int role) const
             case Roles::RoomName:
                 return tr("No preview available");
             case Roles::LastMessage:
-                return QString();
+                return tr("This room is possibly inaccessible");
             case Roles::Time:
                 return QString();
             case Roles::Timestamp:
@@ -275,11 +271,13 @@ RoomlistModel::addRoom(const QString &room_id, bool suppressInsertNotification)
 {
     if (!models.contains(room_id)) {
         // ensure we get read status updates and are only connected once
+        // WORKAROUND(Nico): This is not a lambda, but clazy on alpine currently doesn't
+        // believe us...
         connect(cache::client(),
                 &Cache::roomReadStatus,
                 this,
                 &RoomlistModel::updateReadStatus,
-                Qt::UniqueConnection);
+                Qt::UniqueConnection); // clazy:exclude=lambda-unique-connection
 
         QSharedPointer<TimelineModel> newRoom(new TimelineModel(manager, room_id));
         newRoom->setDecryptDescription(ChatPage::instance()->userSettings()->decryptSidebar());
@@ -344,7 +342,7 @@ RoomlistModel::addRoom(const QString &room_id, bool suppressInsertNotification)
 
             int total_unread_msgs = 0;
 
-            for (const auto &room : qAsConst(models)) {
+            for (const auto &room : std::as_const(models)) {
                 if (!room.isNull() && !room->isSpace())
                     total_unread_msgs += room->notificationCount();
             }
@@ -532,20 +530,23 @@ RoomlistModel::sync(const mtx::responses::Sync &sync_)
         // addRoom will only add the room, if it doesn't exist
         addRoom(qroomid);
         const auto &room_model = models.value(qroomid);
-        room_model->sync(room);
-        // room_model->addEvents(room.timeline);
+
+        // WORKAROUND(Nico): This is not a lambda, but clazy on alpine currently doesn't
+        // believe us
         connect(room_model.data(),
                 &TimelineModel::newCallEvent,
                 ChatPage::instance()->callManager(),
                 &CallManager::syncEvent,
-                Qt::UniqueConnection);
+                Qt::UniqueConnection); // clazy:exclude=lambda-unique-connection
+
+        room_model->sync(room);
 
         if (ChatPage::instance()->userSettings()->typingNotifications()) {
             for (const auto &ev : room.ephemeral.events) {
                 if (auto t =
                       std::get_if<mtx::events::EphemeralEvent<mtx::events::ephemeral::Typing>>(
                         &ev)) {
-                    std::vector<QString> typing;
+                    QStringList typing;
                     typing.reserve(t->content.user_ids.size());
                     for (const auto &user : t->content.user_ids) {
                         if (user != http::client()->user_id().to_string())
@@ -798,11 +799,13 @@ RoomlistModel::setCurrentRoom(const QString &roomid)
             p.roomName_         = QString::fromStdString(i->name);
             p.roomTopic_        = QString::fromStdString(i->topic);
             p.roomAvatarUrl_    = QString::fromStdString(i->avatar_url);
+            p.isFetched_        = true;
             currentRoomPreview_ = std::move(p);
             nhlog::ui()->debug("Switched to (preview): {}",
                                currentRoomPreview_->roomid_.toStdString());
         } else {
             p.roomid_           = roomid;
+            p.isFetched_        = false;
             currentRoomPreview_ = p;
             nhlog::ui()->debug("Switched to (empty): {}",
                                currentRoomPreview_->roomid_.toStdString());
@@ -819,7 +822,7 @@ RoomlistModel::refetchOnlineKeyBackupKeys()
         auto ptr = i.value();
 
         if (!ptr.isNull()) {
-            EventStore::refetchOnlineKeyBackupKeys(ptr.data());
+            ptr->refetchOnlineKeyBackupKeys();
         }
     }
 }
@@ -886,33 +889,54 @@ FilteredRoomlistModel::lessThan(const QModelIndex &left, const QModelIndex &righ
         return a_importance > b_importance;
     }
 
-    // Now sort by recency
+    // Now sort by recency or room name
     // Zero if empty, otherwise the time that the event occured
-    uint64_t a_recency = sourceModel()->data(left_idx, RoomlistModel::Timestamp).toULongLong();
-    uint64_t b_recency = sourceModel()->data(right_idx, RoomlistModel::Timestamp).toULongLong();
 
-    if (a_recency != b_recency)
-        return a_recency > b_recency;
-    else
-        return left.row() < right.row();
+    if (this->sortByAlphabet) {
+        QString a_order = sourceModel()->data(left_idx, RoomlistModel::RoomName).toString();
+        QString b_order = sourceModel()->data(right_idx, RoomlistModel::RoomName).toString();
+
+        auto comp = a_order.compare(b_order, Qt::CaseInsensitive);
+        if (comp != 0)
+            return comp < 0;
+    } else {
+        uint64_t a_order = sourceModel()->data(left_idx, RoomlistModel::Timestamp).toULongLong();
+        uint64_t b_order = sourceModel()->data(right_idx, RoomlistModel::Timestamp).toULongLong();
+
+        if (a_order != b_order)
+            return a_order > b_order;
+    }
+
+    return left.row() < right.row();
 }
 
 FilteredRoomlistModel::FilteredRoomlistModel(RoomlistModel *model, QObject *parent)
   : QSortFilterProxyModel(parent)
   , roomlistmodel(model)
 {
+    instance_ = this;
+
     this->sortByImportance = UserSettings::instance()->sortByImportance();
+    this->sortByAlphabet   = UserSettings::instance()->sortByAlphabet();
     setSourceModel(model);
     setDynamicSortFilter(true);
 
     QObject::connect(UserSettings::instance().get(),
-                     &UserSettings::roomSortingChanged,
+                     &UserSettings::roomSortingChangedImportance,
                      this,
                      [this](bool sortByImportance_) {
                          this->sortByImportance = sortByImportance_;
                          invalidate();
                      });
 
+    QObject::connect(UserSettings::instance().get(),
+                     &UserSettings::roomSortingChangedAlphabetical,
+                     this,
+                     [this](bool sortByAlphabet_) {
+                         this->sortByAlphabet = sortByAlphabet_;
+                         invalidate();
+                     });
+
     connect(roomlistmodel,
             &RoomlistModel::currentRoomChanged,
             this,
@@ -921,6 +945,34 @@ FilteredRoomlistModel::FilteredRoomlistModel(RoomlistModel *model, QObject *pare
     sort(0);
 }
 
+FilteredRoomlistModel *
+FilteredRoomlistModel::create(QQmlEngine *qmlEngine, QJSEngine *)
+{
+    // The instance has to exist before it is used. We cannot replace it.
+    Q_ASSERT(instance_);
+
+    // The engine has to have the same thread affinity as the singleton.
+    Q_ASSERT(qmlEngine->thread() == instance_->thread());
+
+    // There can only be one engine accessing the singleton.
+    static QJSEngine *s_engine = nullptr;
+    if (s_engine)
+        Q_ASSERT(qmlEngine == s_engine);
+    else
+        s_engine = qmlEngine;
+
+    QJSEngine::setObjectOwnership(instance_, QJSEngine::CppOwnership);
+    return instance_;
+}
+
+TimelineModel *
+FilteredRoomlistModel::getRoomById(const QString &id) const
+{
+    auto r = roomlistmodel->getRoomById(id).data();
+    QQmlEngine::setObjectOwnership(r, QQmlEngine::CppOwnership);
+    return r;
+}
+
 void
 FilteredRoomlistModel::updateHiddenTagsAndSpaces()
 {
@@ -929,7 +981,7 @@ FilteredRoomlistModel::updateHiddenTagsAndSpaces()
     hideDMs = false;
 
     auto hidden = UserSettings::instance()->hiddenTags();
-    for (const auto &t : qAsConst(hidden)) {
+    for (const auto &t : std::as_const(hidden)) {
         if (t.startsWith(u"tag:"))
             hiddenTags.push_back(t.mid(4));
         else if (t.startsWith(u"space:"))
@@ -1102,6 +1154,15 @@ FilteredRoomlistModel::filterAcceptsRow(int sourceRow, const QModelIndex &) cons
                       .toBool();
         }
 
+        // If it is a preview but it can't be fetched, it is probably an inaccessible private room.
+        // Hide it if the user isn't an admin.
+        auto index = sourceModel()->index(sourceRow, 0);
+        if (sourceModel()->data(index, RoomlistModel::IsPreview).toBool() &&
+            !sourceModel()->data(index, RoomlistModel::IsPreviewFetched).toBool() &&
+            !Permissions(filterStr).canChange(qml_mtx_events::SpaceChild)) {
+            return false;
+        }
+
         return true;
     } else {
         return true;
@@ -1209,3 +1270,51 @@ FilteredRoomlistModel::previousRoom()
         }
     }
 }
+
+QString
+RoomPreview::inviterAvatarUrl() const
+{
+    if (isInvite_) {
+        auto self = cache::client()->getInviteMember(roomid_.toStdString(),
+                                                     http::client()->user_id().to_string());
+        if (self && !self->inviter.empty()) {
+            auto other = cache::client()->getInviteMember(roomid_.toStdString(), self->inviter);
+            if (other && other->avatar_url.starts_with("mxc://")) {
+                return QString::fromStdString(other->avatar_url);
+            }
+        }
+    }
+
+    return QString();
+}
+QString
+RoomPreview::inviterDisplayName() const
+{
+    if (isInvite_) {
+        auto self = cache::client()->getInviteMember(roomid_.toStdString(),
+                                                     http::client()->user_id().to_string());
+        if (self && !self->inviter.empty()) {
+            auto other = cache::client()->getInviteMember(roomid_.toStdString(), self->inviter);
+            if (other) {
+                return QString::fromStdString(other->name).toHtmlEscaped();
+            }
+        }
+    }
+
+    return QString();
+}
+QString
+RoomPreview::inviterUserId() const
+{
+    if (isInvite_) {
+        auto self = cache::client()->getInviteMember(roomid_.toStdString(),
+                                                     http::client()->user_id().to_string());
+        if (self && !self->inviter.empty()) {
+            return QString::fromStdString(self->inviter);
+        }
+    }
+
+    return QString();
+}
+
+#include "moc_RoomlistModel.cpp"
