@@ -1,41 +1,73 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "TimelineViewManager.h"
 
 #include <QApplication>
+#include <QClipboard>
 #include <QFileDialog>
+#include <QMimeData>
+#include <QQuickItem>
+#include <QQuickTextDocument>
 #include <QStandardPaths>
 #include <QString>
 
 #include "Cache.h"
+#include "Cache_p.h"
 #include "ChatPage.h"
 #include "CombinedImagePackModel.h"
 #include "CommandCompleter.h"
 #include "CompletionProxyModel.h"
 #include "EventAccessors.h"
+#include "GridImagePackModel.h"
 #include "ImagePackListModel.h"
 #include "InviteesModel.h"
 #include "Logging.h"
 #include "MainWindow.h"
 #include "MatrixClient.h"
+#include "MemberList.h"
 #include "RoomsModel.h"
+#include "TimelineModel.h"
 #include "UserSettingsPage.h"
 #include "UsersModel.h"
 #include "Utils.h"
-#include "emoji/EmojiModel.h"
 #include "encryption/VerificationManager.h"
+#include "timeline/CommunitiesModel.h"
+#include "timeline/PresenceEmitter.h"
+#include "timeline/RoomlistModel.h"
+#include "ui/RoomSettings.h"
+#include "ui/UserProfile.h"
 #include "voip/CallManager.h"
 #include "voip/WebRTCSession.h"
 
-namespace msgs = mtx::events::msg;
-
 namespace {
+struct nonesuch
+{
+    ~nonesuch()                      = delete;
+    nonesuch(nonesuch const &)       = delete;
+    void operator=(nonesuch const &) = delete;
+};
+
+namespace detail {
+template<class Default, class AlwaysVoid, template<class...> class Op, class... Args>
+struct detector
+{
+    using value_t = std::false_type;
+    using type    = Default;
+};
+
+template<class Default, template<class...> class Op, class... Args>
+struct detector<Default, std::void_t<Op<Args...>>, Op, Args...>
+{
+    using value_t = std::true_type;
+    using type    = Op<Args...>;
+};
+
+} // namespace detail
+
 template<template<class...> class Op, class... Args>
-using is_detected = typename nheko::detail::detector<nheko::nonesuch, void, Op, Args...>::value_t;
+using is_detected = typename detail::detector<nonesuch, void, Op, Args...>::value_t;
 
 template<class Content>
 using file_t = decltype(Content::file);
@@ -89,7 +121,7 @@ TimelineViewManager::updateColorPalette()
 QColor
 TimelineViewManager::userColor(QString id, QColor background)
 {
-    QPair<QString, quint64> idx{id, background.rgba64()};
+    std::pair<QString, quint64> idx{id, background.rgba64()};
     if (!userColors.contains(idx))
         userColors.insert(idx, QColor(utils::generateContrastingHexColor(id, background)));
     return userColors.value(idx);
@@ -98,29 +130,21 @@ TimelineViewManager::userColor(QString id, QColor background)
 TimelineViewManager::TimelineViewManager(CallManager *, ChatPage *parent)
   : QObject(parent)
   , rooms_(new RoomlistModel(this))
+  , frooms_(new FilteredRoomlistModel(this->rooms_))
   , communities_(new CommunitiesModel(this))
   , verificationManager_(new VerificationManager(this))
   , presenceEmitter(new PresenceEmitter(this))
 {
-    static auto self = this;
-    qmlRegisterSingletonInstance("im.nheko", 1, 0, "TimelineManager", self);
-    qmlRegisterSingletonType<RoomlistModel>(
-      "im.nheko", 1, 0, "Rooms", [](QQmlEngine *, QJSEngine *) -> QObject * {
-          auto ptr = new FilteredRoomlistModel(self->rooms_);
+    instance_ = this;
 
-          connect(self->communities_,
-                  &CommunitiesModel::currentTagIdChanged,
-                  ptr,
-                  &FilteredRoomlistModel::updateFilterTag);
-          connect(self->communities_,
-                  &CommunitiesModel::hiddenTagsChanged,
-                  ptr,
-                  &FilteredRoomlistModel::updateHiddenTagsAndSpaces);
-          return ptr;
-      });
-    qmlRegisterSingletonInstance("im.nheko", 1, 0, "Communities", self->communities_);
-    qmlRegisterSingletonInstance("im.nheko", 1, 0, "VerificationManager", verificationManager_);
-    qmlRegisterSingletonInstance("im.nheko", 1, 0, "Presence", presenceEmitter);
+    connect(this->communities_,
+            &CommunitiesModel::currentTagIdChanged,
+            frooms_,
+            &FilteredRoomlistModel::updateFilterTag);
+    connect(this->communities_,
+            &CommunitiesModel::hiddenTagsChanged,
+            frooms_,
+            &FilteredRoomlistModel::updateHiddenTagsAndSpaces);
 
     updateColorPalette();
 
@@ -157,6 +181,32 @@ TimelineViewManager::TimelineViewManager(CallManager *, ChatPage *parent)
     });
 }
 
+TimelineViewManager *
+TimelineViewManager::create(QQmlEngine *qmlEngine, QJSEngine *)
+{
+    // The instance has to exist before it is used. We cannot replace it.
+    Q_ASSERT(instance_);
+
+    // The engine has to have the same thread affinity as the singleton.
+    Q_ASSERT(qmlEngine->thread() == instance_->thread());
+
+    // There can only be one engine accessing the singleton.
+    static QJSEngine *s_engine = nullptr;
+    if (s_engine)
+        Q_ASSERT(qmlEngine == s_engine);
+    else
+        s_engine = qmlEngine;
+
+    QJSEngine::setObjectOwnership(instance_, QJSEngine::CppOwnership);
+    return instance_;
+}
+
+void
+TimelineViewManager::clearAll()
+{
+    rooms_->clear();
+}
+
 void
 TimelineViewManager::openRoomMembers(TimelineModel *room)
 {
@@ -182,7 +232,10 @@ TimelineViewManager::openRoomSettings(QString room_id)
 void
 TimelineViewManager::openInviteUsers(QString roomId)
 {
-    InviteesModel *model = new InviteesModel{};
+    if (!roomId.startsWith('!'))
+        return;
+
+    InviteesModel *model = new InviteesModel{rooms_->getRoomById(roomId).data()};
     connect(model, &InviteesModel::accept, this, [this, model, roomId]() {
         emit inviteUsers(roomId, model->mxids());
     });
@@ -193,6 +246,9 @@ TimelineViewManager::openInviteUsers(QString roomId)
 void
 TimelineViewManager::openGlobalUserProfile(QString userId)
 {
+    if (!userId.startsWith('@'))
+        return;
+
     UserProfile *profile = new UserProfile{QString{}, userId, this};
     QQmlEngine::setObjectOwnership(profile, QQmlEngine::JavaScriptOwnership);
     emit openProfile(profile);
@@ -220,6 +276,7 @@ TimelineViewManager::sync(const mtx::responses::Sync &sync_)
     this->rooms_->sync(sync_);
     this->communities_->sync(sync_);
     this->presenceEmitter->sync(sync_.presence);
+    this->processIgnoredUsers(sync_.account_data);
 
     if (isInitialSync_) {
         this->isInitialSync_ = false;
@@ -282,7 +339,7 @@ TimelineViewManager::saveMedia(QString mxcUrl)
 {
     const QString downloadsFolder =
       QStandardPaths::writableLocation(QStandardPaths::DownloadLocation);
-    const QString openLocation = downloadsFolder + "/" + mxcUrl.splitRef(u'/').constLast();
+    const QString openLocation = downloadsFolder + "/" + mxcUrl.split(u'/').constLast();
 
     const QString filename = QFileDialog::getSaveFileName(nullptr, {}, openLocation);
 
@@ -320,6 +377,37 @@ TimelineViewManager::saveMedia(QString mxcUrl)
                              });
 }
 
+void
+TimelineViewManager::copyImage(const QString &mxcUrl) const
+{
+    const auto url = mxcUrl.toStdString();
+    QString mimeType;
+
+    http::client()->download(
+      url,
+      [url, mimeType](const std::string &data,
+                      const std::string &,
+                      const std::string &,
+                      mtx::http::RequestErr err) {
+          if (err) {
+              nhlog::net()->warn("failed to retrieve media {}: {} {}",
+                                 url,
+                                 err->matrix_error.error,
+                                 static_cast<int>(err->status_code));
+              return;
+          }
+
+          try {
+              auto img = utils::readImage(QByteArray(data.data(), (qsizetype)data.size()));
+              QGuiApplication::clipboard()->setImage(img);
+
+              return;
+          } catch (const std::exception &e) {
+              nhlog::ui()->warn("Error while copying file to clipboard: {}", e.what());
+          }
+      });
+}
+
 void
 TimelineViewManager::updateReadReceipts(const QString &room_id,
                                         const std::vector<QString> &event_ids)
@@ -415,7 +503,7 @@ TimelineViewManager::focusMessageInput()
     emit focusInput();
 }
 
-QObject *
+QAbstractItemModel *
 TimelineViewManager::completerFor(const QString &completerName, const QString &roomId)
 {
     if (completerName == QLatin1String("user")) {
@@ -424,15 +512,10 @@ TimelineViewManager::completerFor(const QString &completerName, const QString &r
         userModel->setParent(proxy);
         return proxy;
     } else if (completerName == QLatin1String("emoji")) {
-        auto emojiModel = new emoji::EmojiModel();
+        auto emojiModel = new CombinedImagePackModel(roomId.toStdString());
         auto proxy      = new CompletionProxyModel(emojiModel);
         emojiModel->setParent(proxy);
         return proxy;
-    } else if (completerName == QLatin1String("allemoji")) {
-        auto emojiModel = new emoji::EmojiModel();
-        auto proxy      = new CompletionProxyModel(emojiModel, 1, static_cast<size_t>(-1) / 4);
-        emojiModel->setParent(proxy);
-        return proxy;
     } else if (completerName == QLatin1String("room")) {
         auto roomModel = new RoomsModel(false);
         auto proxy     = new CompletionProxyModel(roomModel, 4);
@@ -443,16 +526,12 @@ TimelineViewManager::completerFor(const QString &completerName, const QString &r
         auto proxy     = new CompletionProxyModel(roomModel);
         roomModel->setParent(proxy);
         return proxy;
-    } else if (completerName == QLatin1String("stickers")) {
-        auto stickerModel = new CombinedImagePackModel(roomId.toStdString(), true);
-        auto proxy        = new CompletionProxyModel(stickerModel, 1, static_cast<size_t>(-1) / 4);
-        stickerModel->setParent(proxy);
-        return proxy;
-    } else if (completerName == QLatin1String("customEmoji")) {
-        auto stickerModel = new CombinedImagePackModel(roomId.toStdString(), false);
-        auto proxy        = new CompletionProxyModel(stickerModel);
-        stickerModel->setParent(proxy);
-        return proxy;
+    } else if (completerName == QLatin1String("emojigrid")) {
+        auto stickerModel = new GridImagePackModel(roomId.toStdString(), false);
+        return stickerModel;
+    } else if (completerName == QLatin1String("stickergrid")) {
+        auto stickerModel = new GridImagePackModel(roomId.toStdString(), true);
+        return stickerModel;
     } else if (completerName == QLatin1String("command")) {
         auto commandCompleter = new CommandCompleter();
         auto proxy            = new CompletionProxyModel(commandCompleter);
@@ -463,7 +542,7 @@ TimelineViewManager::completerFor(const QString &completerName, const QString &r
 }
 
 void
-TimelineViewManager::forwardMessageToRoom(mtx::events::collections::TimelineEvents *e,
+TimelineViewManager::forwardMessageToRoom(mtx::events::collections::TimelineEvents const *e,
                                           QString roomId)
 {
     auto room                                                = rooms_->getRoomById(roomId);
@@ -542,9 +621,51 @@ TimelineViewManager::forwardMessageToRoom(mtx::events::collections::TimelineEven
 
 //! WORKAROUND(Nico): for https://bugreports.qt.io/browse/QTBUG-93281
 void
-TimelineViewManager::fixImageRendering(QQuickTextDocument *t, QQuickItem *i)
+TimelineViewManager::fixImageRendering([[maybe_unused]] QQuickTextDocument *t,
+                                       [[maybe_unused]] QQuickItem *i)
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 7, 0)
     if (t) {
         QObject::connect(t->textDocument(), SIGNAL(imagesLoaded()), i, SLOT(updateWholeDocument()));
     }
+#endif
 }
+
+using IgnoredUsers = mtx::events::EphemeralEvent<mtx::events::account_data::IgnoredUsers>;
+
+static QVector<QString>
+convertIgnoredToQt(const IgnoredUsers &ev)
+{
+    QVector<QString> users;
+    for (const mtx::events::account_data::IgnoredUser &user : ev.content.users) {
+        users.push_back(QString::fromStdString(user.id));
+    }
+
+    return users;
+}
+
+QVector<QString>
+TimelineViewManager::getIgnoredUsers()
+{
+    const auto cache = cache::client()->getAccountData(mtx::events::EventType::IgnoredUsers);
+    if (!cache) {
+        return {};
+    }
+
+    return convertIgnoredToQt(std::get<IgnoredUsers>(*cache));
+}
+
+void
+TimelineViewManager::processIgnoredUsers(const mtx::responses::AccountData &data)
+{
+    for (const mtx::events::collections::RoomAccountDataEvents::variant &ev : data.events) {
+        if (!std::holds_alternative<IgnoredUsers>(ev)) {
+            continue;
+        }
+        const auto &ignoredEv = std::get<IgnoredUsers>(ev);
+
+        emit this->ignoredUsersChanged(convertIgnoredToQt(ignoredEv));
+        break;
+    }
+}
+#include "moc_TimelineViewManager.cpp"
