@@ -1,65 +1,88 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
 #include <QHash>
-#include <QQuickItem>
-#include <QQuickTextDocument>
-#include <QWidget>
+#include <QQmlEngine>
 
-#include <mtx/common.hpp>
-#include <mtx/responses/messages.hpp>
+#include <unordered_map>
 
-#include "ReadReceiptsModel.h"
-#include "timeline/CommunitiesModel.h"
-#include "timeline/PresenceEmitter.h"
-#include "timeline/RoomlistModel.h"
-#include "ui/RoomSettings.h"
-#include "ui/UserProfile.h"
+class QQuickItem;
+class QQuickTextDocument;
 
-class MxcImageProvider;
-class BlurhashProvider;
-class ColorImageProvider;
 class UserSettings;
 class ChatPage;
 class ImagePackListModel;
 class TimelineModel;
 class CallManager;
 class VerificationManager;
+class InviteesModel;
+class MemberList;
+class CommunitiesModel;
+class RoomlistModel;
+class PresenceEmitter;
+class UserProfile;
+class RoomSettings;
+class FilteredRoomlistModel;
+class QAbstractItemModel;
 
 namespace mtx::responses {
 struct Sync;
+struct AccountData;
 }
 
-class TimelineViewManager : public QObject
+namespace mtx::events::voip {
+struct CallInvite;
+struct CallCandidates;
+struct CallAnswer;
+struct CallHangUp;
+struct CallSelectAnswer;
+struct CallReject;
+struct CallNegotiate;
+}
+namespace mtx::events::collections {
+struct TimelineEvents;
+}
+
+class TimelineViewManager final : public QObject
 {
     Q_OBJECT
 
+    QML_NAMED_ELEMENT(TimelineManager)
+    QML_SINGLETON
+
     Q_PROPERTY(
       bool isInitialSync MEMBER isInitialSync_ READ isInitialSync NOTIFY initialSyncChanged)
     Q_PROPERTY(bool isConnected READ isConnected NOTIFY isConnectedChanged)
+    Q_PROPERTY(QVector<QString> ignoredUsers READ getIgnoredUsers NOTIFY ignoredUsersChanged)
 
 public:
     TimelineViewManager(CallManager *callManager, ChatPage *parent = nullptr);
 
+    static TimelineViewManager *create(QQmlEngine *qmlEngine, QJSEngine *);
+
+    static TimelineViewManager *instance() { return TimelineViewManager::instance_; }
+
+    QVector<QString> getIgnoredUsers();
+
     void sync(const mtx::responses::Sync &sync_);
 
     VerificationManager *verificationManager() { return verificationManager_; }
 
-    void clearAll() { rooms_->clear(); }
+    void clearAll();
 
     Q_INVOKABLE bool isInitialSync() const { return isInitialSync_; }
     bool isConnected() const { return isConnected_; }
     Q_INVOKABLE void openImageOverlay(TimelineModel *room,
-                                      QString mxcUrl,
-                                      QString eventId,
+                                      const QString &mxcUrl,
+                                      const QString &eventId,
                                       double originalWidth,
                                       double proportionalHeight);
     Q_INVOKABLE void openImagePackSettings(QString roomid);
     Q_INVOKABLE void saveMedia(QString mxcUrl);
+    Q_INVOKABLE void copyImage(const QString &mxcUrl) const;
     Q_INVOKABLE QColor userColor(QString id, QColor background);
     Q_INVOKABLE QString escapeEmoji(QString str) const;
     Q_INVOKABLE QString htmlEscape(QString str) const { return str.toHtmlEscaped(); }
@@ -95,6 +118,7 @@ signals:
                           QString url,
                           double originalWidth,
                           double proportionalHeight);
+    void ignoredUsersChanged(const QVector<QString> &ignoredUsers);
 
 public slots:
     void updateReadReceipts(const QString &room_id, const std::vector<QString> &event_ids);
@@ -109,11 +133,15 @@ public slots:
     void queueCallMessage(const QString &roomid, const mtx::events::voip::CallCandidates &);
     void queueCallMessage(const QString &roomid, const mtx::events::voip::CallAnswer &);
     void queueCallMessage(const QString &roomid, const mtx::events::voip::CallHangUp &);
+    void queueCallMessage(const QString &roomid, const mtx::events::voip::CallSelectAnswer &);
+    void queueCallMessage(const QString &roomid, const mtx::events::voip::CallReject &);
+    void queueCallMessage(const QString &roomid, const mtx::events::voip::CallNegotiate &);
 
     void setVideoCallItem();
 
-    QObject *completerFor(QString completerName, QString roomId = QLatin1String(QLatin1String("")));
-    void forwardMessageToRoom(mtx::events::collections::TimelineEvents *e, QString roomId);
+    QAbstractItemModel *completerFor(const QString &completerName,
+                                     const QString &roomId = QLatin1String(QLatin1String("")));
+    void forwardMessageToRoom(mtx::events::collections::TimelineEvents const *e, QString roomId);
 
     RoomlistModel *rooms() { return rooms_; }
 
@@ -122,19 +150,16 @@ private:
     bool isConnected_   = true;
 
     RoomlistModel *rooms_          = nullptr;
+    FilteredRoomlistModel *frooms_ = nullptr;
     CommunitiesModel *communities_ = nullptr;
 
     // don't move this above the rooms_
     VerificationManager *verificationManager_ = nullptr;
     PresenceEmitter *presenceEmitter          = nullptr;
 
-    QHash<QPair<QString, quint64>, QColor> userColors;
+    QHash<std::pair<QString, quint64>, QColor> userColors;
+
+    inline static TimelineViewManager *instance_ = nullptr;
+
+    void processIgnoredUsers(const mtx::responses::AccountData &data);
 };
-Q_DECLARE_METATYPE(mtx::events::msg::KeyVerificationAccept)
-Q_DECLARE_METATYPE(mtx::events::msg::KeyVerificationCancel)
-Q_DECLARE_METATYPE(mtx::events::msg::KeyVerificationDone)
-Q_DECLARE_METATYPE(mtx::events::msg::KeyVerificationKey)
-Q_DECLARE_METATYPE(mtx::events::msg::KeyVerificationMac)
-Q_DECLARE_METATYPE(mtx::events::msg::KeyVerificationReady)
-Q_DECLARE_METATYPE(mtx::events::msg::KeyVerificationRequest)
-Q_DECLARE_METATYPE(mtx::events::msg::KeyVerificationStart)
