@@ -1,19 +1,18 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
-#include <QAbstractVideoSurface>
+#include <QAudioOutput>
 #include <QBuffer>
-#include <QMediaContent>
 #include <QMediaPlayer>
 #include <QObject>
 #include <QPointer>
+#include <QQuickItem>
 #include <QString>
-
-#include "Logging.h"
+#include <QUrl>
+#include <QVideoSink>
 
 class TimelineModel;
 
@@ -22,15 +21,22 @@ class TimelineModel;
 class MxcMediaProxy : public QMediaPlayer
 {
     Q_OBJECT
+    QML_NAMED_ELEMENT(MxcMedia)
+
     Q_PROPERTY(TimelineModel *roomm READ room WRITE setRoom NOTIFY roomChanged REQUIRED)
     Q_PROPERTY(QString eventId READ eventId WRITE setEventId NOTIFY eventIdChanged)
-    Q_PROPERTY(QAbstractVideoSurface *videoSurface READ getVideoSurface WRITE setVideoSurface NOTIFY
-                 videoSurfaceChanged)
     Q_PROPERTY(bool loaded READ loaded NOTIFY loadedChanged)
     Q_PROPERTY(int orientation READ orientation NOTIFY orientationChanged)
+    Q_PROPERTY(float volume READ volume WRITE setVolume NOTIFY volumeChanged)
+    Q_PROPERTY(bool muted READ muted WRITE setMuted NOTIFY mutedChanged)
 
 public:
     MxcMediaProxy(QObject *parent = nullptr);
+    ~MxcMediaProxy()
+    {
+        stop();
+        this->setSourceDevice(nullptr);
+    }
 
     bool loaded() const { return buffer.size() > 0; }
     QString eventId() const { return eventId_; }
@@ -45,27 +51,48 @@ public:
         room_ = room;
         emit roomChanged();
     }
-    void setVideoSurface(QAbstractVideoSurface *surface);
-    QAbstractVideoSurface *getVideoSurface();
-
     int orientation() const;
 
+    float volume() const { return volume_; }
+    bool muted() const { return muted_; }
+    void setVolume(float val)
+    {
+        volume_ = val;
+        if (auto output = audioOutput()) {
+            output->setVolume(QAudio::convertVolume(
+              val, QAudio::LogarithmicVolumeScale, QAudio::LinearVolumeScale));
+        }
+        emit volumeChanged();
+    }
+    void setMuted(bool val)
+    {
+        muted_ = val;
+        if (auto output = audioOutput()) {
+            output->setMuted(val);
+        }
+        emit mutedChanged();
+    }
+
 signals:
     void roomChanged();
     void eventIdChanged();
     void loadedChanged();
-    void newBuffer(QMediaContent, QIODevice *buf);
+    void newBuffer(QUrl, QIODevice *buf);
 
     void orientationChanged();
-    void videoSurfaceChanged();
 
-private slots:
-    void startDownload();
+    void volumeChanged();
+    void mutedChanged();
+
+public slots:
+    void startDownload(bool onlyCached = false);
 
 private:
     TimelineModel *room_ = nullptr;
     QString eventId_;
     QString filename_;
     QBuffer buffer;
-    QAbstractVideoSurface *m_surface = nullptr;
+    QObject *m_surface = nullptr;
+    float volume_      = 1.f;
+    bool muted_        = false;
 };
