@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -10,11 +8,10 @@
 
 #include "ChatPage.h"
 #include "timeline/InputBar.h"
+#include "timeline/RoomlistModel.h"
 #include "timeline/TimelineModel.h"
 #include "timeline/TimelineViewManager.h"
 
-#include "Logging.h"
-
 NhekoDropArea::NhekoDropArea(QQuickItem *parent)
   : QQuickItem(parent)
 {
@@ -40,6 +37,9 @@ NhekoDropArea::dropEvent(QDropEvent *event)
         auto model = ChatPage::instance()->timelineManager()->rooms()->getRoomById(roomid_);
         if (model) {
             model->input()->insertMimeData(event->mimeData());
+            ChatPage::instance()->timelineManager()->focusMessageInput();
         }
     }
 }
+
+#include "moc_NhekoDropArea.cpp"
