@@ -1,5 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -7,15 +6,22 @@
 
 #include <QApplication>
 #include <QDesktopServices>
+#include <QGuiApplication>
 #include <QStyle>
 #include <QUrl>
+#include <QWindow>
+
+#include <mtx/requests.hpp>
 
 #include "Cache_p.h"
 #include "ChatPage.h"
 #include "Logging.h"
 #include "UserSettingsPage.h"
 #include "Utils.h"
-#include "voip/WebRTCSession.h"
+
+#if XCB_AVAILABLE
+#include <xcb/xproto.h>
+#endif
 
 Nheko::Nheko()
 {
@@ -137,9 +143,9 @@ Nheko::setTransientParent(QWindow *window, QWindow *parentWindow) const
 
 void
 Nheko::createRoom(bool space,
-                  QString name,
-                  QString topic,
-                  QString aliasLocalpart,
+                  const QString &name,
+                  const QString &topic,
+                  const QString &aliasLocalpart,
                   bool isEncrypted,
                   int preset)
 {
@@ -148,7 +154,6 @@ Nheko::createRoom(bool space,
     if (space) {
         req.creation_content       = mtx::events::state::Create{};
         req.creation_content->type = mtx::events::state::room_type::space;
-        req.creation_content->creator.clear();
         req.creation_content->room_version.clear();
     }
 
@@ -177,3 +182,36 @@ Nheko::createRoom(bool space,
 
     emit ChatPage::instance()->createRoom(req);
 }
+
+void
+Nheko::setWindowRole([[maybe_unused]] QWindow *win, [[maybe_unused]] QString newRole) const
+{
+#if XCB_AVAILABLE
+    const QNativeInterface::QX11Application *x11Interface =
+      qGuiApp->nativeInterface<QNativeInterface::QX11Application>();
+
+    if (!x11Interface)
+        return;
+
+    auto connection = x11Interface->connection();
+
+    auto role = newRole.toStdString();
+
+    char WM_WINDOW_ROLE[] = "WM_WINDOW_ROLE";
+    auto cookie = xcb_intern_atom(connection, false, std::size(WM_WINDOW_ROLE) - 1, WM_WINDOW_ROLE);
+    xcb_intern_atom_reply_t *reply = xcb_intern_atom_reply(connection, cookie, nullptr);
+    auto atom                      = reply->atom;
+    free(reply);
+
+    xcb_change_property(connection,
+                        XCB_PROP_MODE_REPLACE,
+                        win->winId(),
+                        atom,
+                        XCB_ATOM_STRING,
+                        8,
+                        role.size(),
+                        role.data());
+#endif
+}
+
+#include "moc_NhekoGlobalObject.cpp"
