@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -9,6 +7,7 @@
 #include <QFontDatabase>
 #include <QObject>
 #include <QPalette>
+#include <QQmlEngine>
 #include <QWindow>
 
 #include "AliasEditModel.h"
@@ -17,10 +16,13 @@
 #include "Theme.h"
 #include "UserProfile.h"
 
-class Nheko final : public QObject
+class Nheko : public QObject
 {
     Q_OBJECT
 
+    QML_ELEMENT
+    QML_SINGLETON
+
     Q_PROPERTY(QPalette colors READ colors NOTIFY colorsChanged)
     Q_PROPERTY(QPalette inactiveColors READ inactiveColors NOTIFY colorsChanged)
     Q_PROPERTY(Theme theme READ theme NOTIFY colorsChanged)
