@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -17,7 +15,6 @@
 
 #include "Cache.h"
 #include "Cache_p.h"
-#include "Config.h"
 #include "Logging.h"
 #include "MatrixClient.h"
 #include "Utils.h"
@@ -76,6 +73,11 @@ RoomSettings::RoomSettings(QString roomid, QObject *parent)
     guestRules_ = info_.guest_access ? AccessState::CanJoin : AccessState::Forbidden;
     emit accessJoinRulesChanged();
 
+    if (auto ev = cache::client()->getStateEvent<mtx::events::state::HistoryVisibility>(
+          roomid_.toStdString())) {
+        this->historyVisibility_ = ev->content.history_visibility;
+    }
+
     this->allowedRoomsModel = new RoomSettingsAllowedRoomsModel(this);
 }
 
@@ -153,6 +155,22 @@ RoomSettings::notifications()
     return notifications_;
 }
 
+RoomSettings::Visibility
+RoomSettings::historyVisibility() const
+{
+    switch (this->historyVisibility_) {
+    case mtx::events::state::Visibility::WorldReadable:
+        return WorldReadable;
+    case mtx::events::state::Visibility::Joined:
+        return Joined;
+    case mtx::events::state::Visibility::Invited:
+        return Invited;
+    case mtx::events::state::Visibility::Shared:
+        return Shared;
+    }
+    return Shared;
+}
+
 bool
 RoomSettings::privateAccess() const
 {
@@ -280,6 +298,20 @@ RoomSettings::canChangeAvatar() const
     return false;
 }
 
+bool
+RoomSettings::canChangeHistoryVisibility() const
+{
+    try {
+        return cache::hasEnoughPowerLevel({EventType::RoomHistoryVisibility},
+                                          roomid_.toStdString(),
+                                          utils::localUser().toStdString());
+    } catch (const lmdb::error &e) {
+        nhlog::db()->warn("lmdb error: {}", e.what());
+    }
+
+    return false;
+}
+
 bool
 RoomSettings::isEncryptionEnabled() const
 {
@@ -459,6 +491,52 @@ RoomSettings::changeName(const QString &name)
       });
 }
 
+void
+RoomSettings::changeHistoryVisibility(Visibility value)
+{
+    auto tempVis = mtx::events::state::Visibility::Shared;
+
+    switch (value) {
+    case WorldReadable:
+        tempVis = mtx::events::state::Visibility::WorldReadable;
+        break;
+    case Joined:
+        tempVis = mtx::events::state::Visibility::Joined;
+        break;
+    case Invited:
+        tempVis = mtx::events::state::Visibility::Invited;
+        break;
+    case Shared:
+        tempVis = mtx::events::state::Visibility::Shared;
+        break;
+    default:
+        return;
+    }
+
+    using namespace mtx::events;
+    auto proxy = std::make_shared<ThreadProxy>();
+    connect(proxy.get(), &ThreadProxy::eventSent, this, [this, tempVis]() {
+        this->historyVisibility_ = tempVis;
+        emit historyVisibilityChanged();
+    });
+    connect(proxy.get(), &ThreadProxy::error, this, &RoomSettings::displayError);
+
+    state::HistoryVisibility body;
+    body.history_visibility = tempVis;
+
+    http::client()->send_state_event(
+      roomid_.toStdString(),
+      body,
+      [proxy](const mtx::responses::EventId &, mtx::http::RequestErr err) {
+          if (err) {
+              emit proxy->error(QString::fromStdString(err->matrix_error.error));
+              return;
+          }
+
+          emit proxy->eventSent();
+      });
+}
+
 void
 RoomSettings::changeTopic(const QString &topic)
 {
@@ -649,7 +727,7 @@ RoomSettingsAllowedRoomsModel::RoomSettingsAllowedRoomsModel(RoomSettings *paren
 
     this->listedRoomIds = QStringList(parentSpaces.begin(), parentSpaces.end());
 
-    for (const auto &e : qAsConst(this->allowedRoomIds)) {
+    for (const auto &e : std::as_const(this->allowedRoomIds)) {
         if (!this->parentSpaces.count(e))
             this->listedRoomIds.push_back(e);
     }
@@ -733,3 +811,5 @@ RoomSettings::applyAllowedFromModel()
     this->allowedRoomsModified_ = true;
     emit allowedRoomsModifiedChanged();
 }
+
+#include "moc_RoomSettings.cpp"
