@@ -1,5 +1,4 @@
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -8,6 +7,7 @@
 #include <optional>
 
 #include <QObject>
+#include <QQmlEngine>
 
 #include <mtx/responses/public_rooms.hpp>
 
@@ -26,6 +26,9 @@ class RoomSummary final : public QObject
 {
     Q_OBJECT
 
+    QML_ELEMENT
+    QML_UNCREATABLE("Please use joinRoom to create a room summary.")
+
     Q_PROPERTY(QString reason READ reason WRITE setReason NOTIFY reasonChanged)
 
     Q_PROPERTY(QString roomid READ roomid NOTIFY loaded)
